import inspect
import logging
import sys
from typing import TYPE_CHECKING, Optional

import loguru

from aioarxiv.config import ArxivConfig, default_config

if TYPE_CHECKING:
    # avoid sphinx autodoc resolve annotation failed
    # because loguru module do not have `Logger` class actually
    from loguru import Logger, Record

logger: "Logger" = loguru.logger
"""日志记录器对象。

default:

- 格式: `[%(asctime)s %(name)s] %(levelname)s: %(message)s`
- 等级: `INFO` ,  根据 `config.log_level` 配置改变
- 输出: 输出至 stdout

usage:
    ```python
    from log import logger
    ```
"""

_current_config: Optional[ArxivConfig] = None


def set_config(config: ArxivConfig) -> None:
    """设置当前全局配置"""
    global _current_config
    _current_config = config


def get_config() -> ArxivConfig:
    """获取当前配置或默认配置"""
    return _current_config or default_config


# https://loguru.readthedocs.io/en/stable/overview.html#entirely-compatible-with-standard-logging
class LoguruHandler(logging.Handler):  # pragma: no cover
    """logging 与 loguru 之间的桥梁,  将 logging 的日志转发到 loguru。"""

    def emit(self, record: logging.LogRecord) -> None:
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        frame, depth = inspect.currentframe(), 0
        while frame and (depth == 0 or frame.f_code.co_filename == logging.__file__):
            frame = frame.f_back
            depth += 1

        logger.opt(depth=depth, exception=record.exc_info).log(
            level,
            record.getMessage(),
        )


def default_filter(record: "Record"):
    """默认的日志过滤器,  根据 `config.log_level` 配置改变日志等级。"""
    log_level = record["extra"].get("arxiv_log_level")

    if log_level is None:
        config = get_config()
        log_level = config.log_level if config else default_config.log_level

    levelno = logger.level(log_level).no if isinstance(log_level, str) else log_level
    return record["level"].no >= levelno


default_format: str = (
    "<g>{time:MM-DD HH:mm:ss}</g> "
    "[<lvl>{level}</lvl>] "
    "<c><u>{name}</u></c> | "
    "<c>{function}:{line}</c>| "
    "{message}"
)
"""默认日志格式"""

logger.remove()
logger_id = logger.add(
    sys.stdout,
    level=0,
    diagnose=False,
    filter=default_filter,
    format=default_format,
)
"""默认日志处理器 id"""

__autodoc__ = {"logger_id": False}



from datetime import datetime, timedelta
from pathlib import Path
from uuid import UUID
from zoneinfo import ZoneInfo

import pytest
from yarl import URL

from aioarxiv.client.arxiv_client import ArxivClient
from aioarxiv.models import (
    Metadata,
)


@pytest.mark.asyncio
async def test_client_initialization(mock_config):
    """测试客户端初始化"""
    client = ArxivClient()
    assert client._config == mock_config
    assert client._enable_downloader is False
    assert client.download_dir is None

    custom_config = mock_config.model_copy(update={"page_size": 10})
    download_dir = Path("./downloads")
    client = ArxivClient(
        config=custom_config, enable_downloader=True, download_dir=download_dir
    )
    assert client._config == custom_config
    assert client._enable_downloader is True
    assert client.download_dir == download_dir


@pytest.mark.asyncio
async def test_build_search_metadata(
    mock_arxiv_client, sample_search_result, sample_paper, mock_config
):
    """测试搜索元数据构建"""
    # 创建一个新的元数据对象，确保 source 是 URL 类型
    metadata = Metadata(
        start_time=datetime.now(tz=ZoneInfo(mock_config.timezone)),
        end_time=datetime.now(tz=ZoneInfo(mock_config.timezone)),
        missing_results=0,
        pagesize=10,
        source=URL("http://export.arxiv.org/api/query"),
    )

    # 更新 search_result 的元数据
    search_result = sample_search_result.model_copy(update={"metadata": metadata})

    updated_result = mock_arxiv_client._build_search_result_metadata(
        search_result, page=1, batch_size=10, papers=[sample_paper]
    )

    assert len(updated_result.papers) == 1
    assert updated_result.page == 1
    assert updated_result.has_next is False
    assert updated_result.metadata.pagesize == mock_arxiv_client._config.page_size
    assert isinstance(updated_result.metadata.source, URL)


@pytest.mark.asyncio
async def test_metadata_duration_calculation(mock_datetime):
    """测试元数据持续时间计算"""
    start_time = mock_datetime
    end_time = mock_datetime + timedelta(seconds=1)

    metadata = Metadata(
        start_time=start_time,
        end_time=end_time,
        missing_results=0,
        pagesize=10,
        source=URL("http://test.com"),
    )

    assert metadata.duration_seconds == 1.000
    assert metadata.duration_ms == 1000.000


# @pytest.mark.asyncio
# async def test_search_with_params(
#     mock_arxiv_client, mock_response, mock_session_manager, mock_config
# ):
#     """测试带参数的搜索"""
#     mock_session_manager.request.return_value = mock_response
#
#     params = {
#         "query": "neural networks",
#         "max_results": 5,
#         "sort_by": SortCriterion.SUBMITTED,
#         "sort_order": SortOrder.ASCENDING,
#     }
#
#     result = await mock_arxiv_client.search(**params)
#
#     assert result.total_result == 218712
#
#     paper = result.papers[0]
#     assert paper.info.id == "0102536v1"
#     assert (
#         paper.info.title
#         == "Impact of Electron-Electron Cusp on Configuration Interaction Energies"
#     )
#
#     authors = paper.info.authors
#     assert len(authors) == 5
#     assert authors[0].name == "David Prendergast"
#     assert authors[0].affiliation == "Department of Physics"
#     assert authors[1].name == "M. Nolan"
#     assert authors[1].affiliation == "NMRC, University College, Cork, Ireland"
#
#     assert paper.doi == "10.1063/1.1383585"
#     assert paper.journal_ref == "J. Chem. Phys. 115, 1626 (2001)"
#     assert "11 pages, 6 figures, 3 tables" in paper.comment
#     assert paper.info.categories.primary.term == "cond-mat.str-el"
#
#     call_args = mock_session_manager.request.call_args
#     assert call_args is not None
#     _, kwargs = call_args
#
#     query_params = kwargs["params"]
#     assert query_params["search_query"] == "neural networks"
#     assert query_params["max_results"] == mock_config.page_size
#     assert query_params["sortBy"] == SortCriterion.SUBMITTED.value
#     assert query_params["sortOrder"] == SortOrder.ASCENDING.value


def test_search_result_computed_fields(sample_search_result):
    """测试搜索结果的计算字段"""
    assert sample_search_result.papers_count == 1
    assert isinstance(sample_search_result.id, UUID)


@pytest.mark.asyncio
async def test_client_context_manager(mock_arxiv_client):
    """测试客户端上下文管理器"""
    async with mock_arxiv_client as c:
        assert isinstance(c, ArxivClient)

    mock_arxiv_client._session_manager.close.assert_awaited_once()



import asyncio
from datetime import datetime
from pathlib import Path
from typing import Optional
from zoneinfo import ZoneInfo

import aiofiles
from platformdirs import user_documents_path
from tenacity import retry, stop_after_attempt, wait_exponential

from aioarxiv.config import ArxivConfig, default_config
from aioarxiv.exception import PaperDownloadException
from aioarxiv.models import Paper, SearchResult
from aioarxiv.utils import format_datetime, log_retry_attempt, sanitize_title
from aioarxiv.utils.log import logger
from aioarxiv.utils.session import SessionManager


class DownloadTracker:
    """A tracker that manages and monitors batch download progress for research papers.

    This class maintains the state of a batch download operation, including success,
    failure counts, and timing information. It provides methods to update progress
    and access statistics about the download operation.

    Args:
        total (int): The total number of papers to be downloaded in this batch.

    Usage:
        ```python
        # Create a tracker for downloading 10 papers
        tracker = DownloadTracker(total=10)

        # Update progress as downloads complete
        tracker.add_completed()

        # Record failed downloads with error information
        try:
            await download_paper(paper)
        except Exception as e:
            tracker.add_failed(paper, e)

        # Check progress
        progress = tracker.progress  # returns percentage complete
        failed_papers = tracker.failed_papers  # get list of failed downloads
        ```
    """

    def __init__(self, total: int) -> None:
        self.total: int = total
        self.completed: int = 0
        self.failed: int = 0
        self.start_time: datetime = datetime.now(ZoneInfo(default_config.timezone))
        self.end_time: Optional[datetime] = None
        self._failed_papers: list[tuple[Paper, Exception]] = []

    def add_failed(self, paper: Paper, error: Exception) -> None:
        """Records a paper that failed to download.

        Args:
            paper (Paper): The paper object that failed to download.
            error (Exception): The exception that occurred during download.
        """
        self.failed += 1
        self._failed_papers.append((paper, error))

    def add_completed(self) -> None:
        """Records a successfully completed paper download."""
        self.completed += 1

    @property
    def progress(self) -> float:
        """The current download progress as a percentage.

        Returns:
            float: The percentage of papers processed (completed + failed).
        """
        return (self.completed + self.failed) / self.total * 100

    @property
    def failed_papers(self) -> list[tuple[Paper, Exception]]:
        """List of papers that failed to download with their corresponding errors.

        Returns:
            list[tuple[Paper, Exception]]: A list of tuples containing the failed
                paper objects and their associated error exceptions.
        """
        return self._failed_papers


class ArxivDownloader:
    """A concurrent downloader for arXiv research papers.

    This class manages the asynchronous download of arXiv papers, handling concurrent
    requests, file management, and error recovery. It supports batch downloads with
    automatic retry mechanisms and progress tracking.

    Args:
        session_manager (Optional[SessionManager]): Custom session manager for requests.
        download_dir (Optional[Path]): Custom directory for downloaded papers.
        config (Optional[ArxivConfig]): Configuration for the downloader.

    Usage:
        ```python
        # Create a downloader with default settings
        downloader = ArxivDownloader()

        # Download a single paper
        await downloader.download_paper(paper)

        # Batch download papers from search results
        search_result = await arxiv.search("quantum computing")
        tracker = await downloader.batch_download(search_result)

        # Check download statistics
        print(f"Downloaded: {tracker.completed}")
        print(f"Failed: {tracker.failed}")
        ```
    """

    def __init__(
        self,
        session_manager: Optional[SessionManager] = None,
        download_dir: Optional[Path] = None,
        # cache_dir: Optional[Path] = None,
        config: Optional[ArxivConfig] = None,
    ) -> None:
        self._session_manager = session_manager
        self._download_dir = download_dir
        # self._cache_dir = cache_dir
        self._config = config or default_config
        self._semaphore = asyncio.Semaphore(self._config.max_concurrent_requests)

    @property
    def session_manager(self) -> SessionManager:
        """The session manager used for HTTP requests.

        Returns:
            SessionManager: The current session manager instance, creating a new one
                if none exists.
        """
        if self._session_manager is None:
            self._session_manager = SessionManager()
        return self._session_manager

    @property
    def download_dir(self) -> Path:
        """The directory where downloaded papers are saved.

        Returns:
            Path: The path to the download directory, creating it if it doesn't exist.
        """
        if self._download_dir is None:
            self._download_dir = user_documents_path() / "aioarxiv"
        self._download_dir.mkdir(parents=True, exist_ok=True)
        return self._download_dir

    # @property
    # def cache_dir(self) -> Path:
    #     """缓存目录"""
    #     if self._cache_dir is None:
    #         self._cache_dir = user_cache_path("aioarxiv")
    #     self._cache_dir.mkdir(parents=True, exist_ok=True)
    #     return self._cache_dir

    @staticmethod
    def file_name(paper: Paper) -> str:
        """Generates a sanitized filename for a paper.

        Args:
            paper (Paper): The paper object to generate a filename for.

        Returns:
            str: A sanitized filename including the paper title and update date.
        """
        file_name = f"{paper.info.title} {format_datetime(paper.info.updated)}"
        return f"{sanitize_title(file_name)}.pdf"

    def _prepare_paths(
        self, paper: Paper, filename: Optional[str] = None
    ) -> tuple[Path, Path]:
        """
        Prepare file paths for downloading a paper.

        Args:
            paper (Paper): The paper object to download.
            filename (Optional[str]): Custom filename for the downloaded paper.

        Returns:
            tuple[Path, Path]: The final file path and temporary file path.
        """
        name = filename or self.file_name(paper)
        file_path = self.download_dir / name
        temp_path = file_path.with_suffix(".tmp")
        return file_path, temp_path

    async def _download_to_temp(self, url: str, temp_path: Path) -> None:
        """
        Download a file to a temporary location.

        Args:
            url (str): The URL to download the file from.
            temp_path (Path): The temporary file path to save the downloaded content.
        """
        async with self._semaphore:
            response = await self.session_manager.request("GET", url)
            if response.status != 200:
                raise PaperDownloadException(f"HTTP status {response.status}")

            async with aiofiles.open(temp_path, "wb") as f:
                async for chunk in response.content.iter_chunked(8192):
                    await f.write(chunk)  # type: ignore

    async def _download_with_context(
        self, paper: Paper, context: DownloadTracker
    ) -> None:
        """
        Download a paper with a download context.

        Args:
            paper (Paper): The paper object to download.
            context (DownloadTracker): The download context
        """
        try:
            await self.download_paper(paper, f"{paper.info.id}.pdf")
            context.add_completed()
        except Exception as e:
            context.add_failed(paper, e)
            logger.error(
                f"论文 {paper.info.id} 下载失败: {e!s}",
                extra={"paper_id": paper.info.id},
            )

    @retry(
        stop=stop_after_attempt(3),
        wait=wait_exponential(multiplier=1, min=4, max=10),
        retry_error_callback=log_retry_attempt,
    )
    async def download_paper(
        self, paper: Paper, filename: Optional[str] = None
    ) -> None:
        """
        Download a single paper.

        Args:
            paper (Paper): The paper object to download.
            filename (Optional[str]): Custom filename for the downloaded paper.

        Raises:
            PaperDownloadException: If the download fails.
        """
        file_path, temp_path = self._prepare_paths(paper, filename)
        logger.info(f"开始下载论文: {paper.pdf_url}")

        try:
            await self._download_to_temp(str(paper.pdf_url), temp_path)
            temp_path.rename(file_path)
            logger.info(f"下载完成: {file_path}")
            file_size = file_path.stat().st_size
            logger.debug(f"文件大小: {file_size / 1024:.1f} KB")
        except Exception as e:
            logger.error(f"下载失败: {e}")
            temp_path.unlink(missing_ok=True)
            file_path.unlink(missing_ok=True)
            raise PaperDownloadException(f"下载失败: {e}") from e

    async def batch_download(
        self,
        search_result: SearchResult,
    ) -> DownloadTracker:
        """
        Batch download papers from a search result.

        Args:
            search_result (SearchResult): The search result object to download papers.

        Returns:
            DownloadTracker: The download context with progress and statistics.
        """
        context = DownloadTracker(len(search_result.papers))
        tasks = []

        for paper in search_result.papers:
            if paper.pdf_url:
                tasks.append(
                    asyncio.create_task(self._download_with_context(paper, context))
                )
            else:
                context.add_failed(paper, ValueError("No PDF URL available"))

        await asyncio.gather(*tasks)
        context.end_time = datetime.now(ZoneInfo(default_config.timezone))
        return context



from datetime import datetime
from typing import ClassVar, Optional, cast
import xml.etree.ElementTree as ET
from zoneinfo import ZoneInfo

from aiohttp import ClientResponse
from pydantic import AnyUrl, HttpUrl
from yarl import URL

from aioarxiv.config import default_config
from aioarxiv.exception import ParserException
from aioarxiv.models import (
    Author,
    BasicInfo,
    Category,
    Metadata,
    Paper,
    PrimaryCategory,
    SearchParams,
    SearchResult,
)

from . import create_parser_exception
from .log import logger


class ArxivParser:
    """
    arXiv API响应解析器

    Attributes:
        NS (ClassVar[dict[str, str]]): XML命名空间

    Args:
        response_context: API响应内容

    Raises:
        ParserException: 如果解析失败
    """

    NS: ClassVar[dict[str, str]] = {
        "atom": "http://www.w3.org/2005/Atom",
        "opensearch": "http://a9.com/-/spec/opensearch/1.1/",
        "arxiv": "http://arxiv.org/schemas/atom",
    }

    def __init__(self, response_context: str, raw_response: ClientResponse) -> None:
        self.response_context = response_context
        self.raw_response = raw_response
        self.entry = ET.fromstring(response_context)  # Root element of the XML tree

    @staticmethod
    def build_paper(
        data: ET.Element,
    ) -> Paper:
        """统一处理论文解析"""
        parser = PaperParser(data)
        basic_info = parser.parse_basics_info()
        return Paper(
            info=basic_info,
            pdf_url=parser.parse_pdf_url(),
            **parser.parse_optional_fields(),
        )

    def _parse_root(self) -> list[Paper]:
        """
        解析根元素

        Returns:
            list[Paper]: 论文列表
        """
        papers = []
        for i, entry in enumerate(self.entry.findall("atom:entry", ArxivParser.NS)):
            try:
                papers.append(self.build_paper(entry))
            except Exception as e:
                raise create_parser_exception(
                    entry,
                    str(self.raw_response.url),
                    message=f"解析第 {i + 1} 篇论文失败",
                    namespace=ArxivParser.NS["atom"],
                    error=e,
                ) from e

        return papers

    def parse_feed(self) -> list[Paper]:
        """
        解析arXiv API的Atom feed内容

        Returns:
            list[Paper]: 论文列表
        """
        try:
            papers = self._parse_root()
            logger.trace(f"Parsed {len(papers)} papers")
            return papers
        except ET.ParseError as e:
            raise create_parser_exception(
                self.entry,
                str(self.raw_response.url),
                message="解析失败",
                error=e,
            ) from e
        except ParserException:
            raise

    def parse_total_result(self) -> int:
        """
        解析总结果数

        Returns:
            int: 总结果数

        Raises:
            ParserException: 如果解析失败
        """
        total_element = self.entry.find("opensearch:totalResults", ArxivParser.NS)
        if total_element is None or total_element.text is None:
            raise create_parser_exception(
                self.entry,
                "",
                message="缺少总结果数元素",
            )

        return int(total_element.text)

    def build_search_result(self, query_params: SearchParams) -> SearchResult:
        return SearchResult(
            papers=self.parse_feed(),
            total_result=self.parse_total_result(),
            page=1,
            has_next=False,
            query_params=query_params,
            metadata=Metadata(
                missing_results=0,
                pagesize=0,
                source=URL(self.raw_response.url),
                end_time=None,
            ),
        )


class PaperParser:
    """Paper解析器"""

    def __init__(self, entry: ET.Element) -> None:
        self.entry = entry

    def parse_authors(self) -> list[Author]:
        """
        解析作者信息

        Returns:
            list[Author]: 作者列表
        """

        def get_text(element: ET.Element, tag: str, namespace: dict) -> Optional[str]:
            """辅助函数,  用于获取子元素的文本内容"""
            sub_elem = element.find(tag, namespace)
            return sub_elem.text if sub_elem is not None else None

        authors = []
        for author_elem in self.entry.findall("atom:author", ArxivParser.NS):
            name = get_text(author_elem, "atom:name", ArxivParser.NS)
            affiliation = get_text(author_elem, "arxiv:affiliation", ArxivParser.NS)

            if name:
                authors.append(Author(name=name, affiliation=affiliation))

        if not authors:
            raise create_parser_exception(
                self.entry,
                "",
                message="缺少作者信息",
            )

        logger.trace(f"作者信息: {authors}")
        return authors

    def parse_categories(self) -> Category:
        """
        解析分类信息

        Returns:
            Category: 分类信息
        """

        def parse_primary() -> PrimaryCategory:
            primary_elem = self.entry.find("arxiv:primary_category", ArxivParser.NS)
            if primary_elem is None:
                raise create_parser_exception(self.entry, "", message="缺少主分类信息")
            return PrimaryCategory(
                term=primary_elem.get("term", ""),
                scheme=cast(AnyUrl, primary_elem.attrib.get("scheme")),
                label=primary_elem.get("label"),
            )

        def parse_secondary(primary_term: str) -> list[str]:
            categories = []
            for cat in self.entry.findall("category", ArxivParser.NS):
                term = cat.get("term")
                if term and term != primary_term:
                    categories.append(term)
            return categories

        primary = parse_primary()
        secondary = parse_secondary(primary.term)

        return Category(primary=primary, secondary=secondary)

    def parse_basics_info(self) -> BasicInfo:
        """
        解析基础信息

        Returns:
            BasicInfo: 基础信息
        """

        def get_or_raise(element: ET.Element, tag: str) -> str:
            sub_elem = element.find(f"atom:{tag}", ArxivParser.NS)
            if sub_elem is None or sub_elem.text is None:
                raise create_parser_exception(
                    element,
                    "",
                    message=f"缺少 {tag} 元素",
                )
            return sub_elem.text

        return BasicInfo(
            id=get_or_raise(self.entry, "id").split("/")[-1],
            title=get_or_raise(self.entry, "title"),
            summary=get_or_raise(self.entry, "summary"),
            authors=self.parse_authors(),
            categories=self.parse_categories(),
            published=self.parse_datetime(get_or_raise(self.entry, "published")),
            updated=self.parse_datetime(get_or_raise(self.entry, "updated")),
        )

    def parse_pdf_url(self) -> Optional[HttpUrl]:
        """
        解析PDF链接

        Returns:
            Optional[str]: PDF链接或None
        """
        try:
            links = self.entry.findall("atom:link", ArxivParser.NS)
            if not links:
                logger.warning("未找到任何链接")
                return None

            pdf_url = next(
                (
                    link.attrib["href"]
                    for link in links
                    if link.attrib.get("type") == "application/pdf"
                ),
                None,
            )

            if pdf_url is None:
                logger.warning("未找到PDF链接")
                return None

            return cast(HttpUrl, pdf_url)  # stupid type checker

        except (KeyError, AttributeError) as e:
            raise create_parser_exception(
                self.entry,
                "PDF链接解析失败",
                namespace=ArxivParser.NS["atom"],
                error=e,
            ) from e

    def parse_optional_fields(self) -> dict[str, Optional[str]]:
        """
        解析可选字段

        Returns:
            dict: 可选字段字典
        """
        fields = {
            "comment": self.entry.find("arxiv:comment", ArxivParser.NS),
            "journal_ref": self.entry.find("arxiv:journal_ref", ArxivParser.NS),
            "doi": self.entry.find("arxiv:doi", ArxivParser.NS),
        }

        return {k: v.text if v is not None else None for k, v in fields.items()}

    @staticmethod
    def parse_datetime(date_str: str) -> datetime:
        """
        解析ISO格式的日期时间字符串

        Args:
            date_str: ISO格式的日期时间字符串

        Returns:
            datetime: 解析后的datetime对象

        Raises:
            ValueError: 日期格式无效
        """
        try:
            normalized_date = date_str.replace("Z", "+00:00")
            dt = datetime.fromisoformat(normalized_date)
            return dt.replace(tzinfo=ZoneInfo(default_config.timezone))
        except ValueError as e:
            msg = f"日期格式: {date_str} 不符合预期"
            raise ValueError(msg) from e



import asyncio
import math
from typing import NoReturn

import pytest
from pytest_mock import MockerFixture

from aioarxiv.utils.rate_limiter import RateLimiter


@pytest.fixture
def limiter() -> RateLimiter:
    """Create a RateLimiter instance for testing.

    Returns:
        RateLimiter: A fresh RateLimiter instance with test settings.
    """
    return RateLimiter(calls=2, period=1.0)  # 使用较小的值便于测试


@pytest.mark.asyncio
async def test_basic_rate_limiting(mocker: MockerFixture, limiter: RateLimiter) -> None:
    """Test basic rate limiting functionality.

    Args:
        mocker: pytest-mock fixture
        limiter: The rate limiter fixture
    """
    current_time = 0.0
    sleep_calls: list[float] = []

    # 模拟时间和睡眠函数
    def mock_time() -> float:
        return current_time

    async def mock_sleep(duration: float) -> None:
        nonlocal current_time
        sleep_calls.append(duration)
        current_time += duration

    mocker.patch("time.time", mock_time)
    mocker.patch("asyncio.sleep", mock_sleep)

    call_order: list[int] = []

    @limiter.limit()
    async def test_func() -> int:
        call_number = len(call_order) + 1
        call_order.append(call_number)
        return call_number

    # 执行测试
    results = await asyncio.gather(*(test_func() for _ in range(3)))

    # 验证结果
    assert results == [1, 2, 3], "函数调用应该按顺序执行"
    assert (
        len(sleep_calls) == 1
    ), f"应该只有一次速率限制延迟，实际有 {len(sleep_calls)} 次"
    if sleep_calls:
        assert math.isclose(
            sleep_calls[0], 1.0, rel_tol=0.05
        ), f"延迟时间应接近 1.0 秒，实际为 {sleep_calls[0]:.2f} 秒"


@pytest.mark.asyncio
async def test_window_sliding(mocker: MockerFixture, limiter: RateLimiter) -> None:
    """Test time window sliding behavior.

    Args:
        mocker: pytest-mock fixture
        limiter: The rate limiter fixture
    """
    current_time = 0.0
    sleep_calls: list[float] = []

    async def mock_sleep(duration: float) -> None:
        nonlocal current_time
        sleep_calls.append(duration)
        current_time += duration

    def mock_time() -> float:
        return current_time

    mocker.patch("time.time", mock_time)
    mocker.patch("asyncio.sleep", mock_sleep)

    @limiter.limit()
    async def test_func() -> None:
        pass

    # 执行测试
    await test_func()  # 第一次调用
    await test_func()  # 第二次调用（应该立即执行）
    await test_func()  # 第三次调用（应该被限制）

    assert len(sleep_calls) == 1, "应该只有一次速率限制延迟"
    if sleep_calls:
        assert math.isclose(
            sleep_calls[0], 1.0, rel_tol=0.05
        ), f"延迟时间应接近 1.0 秒，实际为 {sleep_calls[0]:.2f} 秒"


@pytest.mark.asyncio
async def test_concurrent_requests():
    limiter = RateLimiter(calls=3, period=0.5)

    semaphore = asyncio.Semaphore(2)
    active_count = 0
    max_active = 0

    @limiter.limit()
    async def test_func() -> None:
        nonlocal active_count, max_active
        async with semaphore:
            active_count += 1
            max_active = max(max_active, active_count)
            await asyncio.sleep(0.01)
            active_count -= 1

    tasks = [test_func() for _ in range(5)]
    await asyncio.gather(*tasks)

    assert max_active <= 2


@pytest.mark.asyncio
async def test_error_handling(limiter: RateLimiter) -> None:
    """Test error propagation.

    Args:
        limiter: The rate limiter fixture
    """

    @limiter.limit()
    async def failing_func() -> NoReturn:
        raise ValueError("测试错误")

    with pytest.raises(ValueError, match="测试错误"):
        await failing_func()


@pytest.mark.asyncio
async def test_multiple_limiters() -> None:
    """Test independent rate limiters.

    This test verifies that:
    1. Different rate limiter instances are independent
    2. Each limiter maintains its own state
    """
    limiter_a = RateLimiter(calls=2, period=1.0)
    limiter_b = RateLimiter(calls=2, period=1.0)

    calls_a = []
    calls_b = []

    @limiter_a.limit()
    async def func_a() -> None:
        calls_a.append(asyncio.get_event_loop().time())
        await asyncio.sleep(0.1)

    @limiter_b.limit()
    async def func_b() -> None:
        calls_b.append(asyncio.get_event_loop().time())
        await asyncio.sleep(0.1)

    # Test interleaved execution
    await func_a()
    await func_b()
    await asyncio.sleep(1.1)  # Wait for window to reset
    await func_a()
    await func_b()

    assert len(calls_a) == 2, "Should complete two calls to func_a"
    assert len(calls_b) == 2, "Should complete two calls to func_b"

    # Verify timing between calls
    timestamps = sorted(calls_a + calls_b)
    assert timestamps[2] - timestamps[0] >= 1.0



from collections.abc import AsyncGenerator
from typing import Any

import pytest
from pytest_mock import MockerFixture

from aioarxiv.client.downloader import ArxivDownloader
from aioarxiv.models import Paper


class MockResponse:
    """模拟HTTP响应对象"""

    def __init__(self, status: int = 200, content: bytes = b"mock_pdf_content") -> None:
        """
        初始化mock响应

        Args:
            status: HTTP状态码
            content: 响应内容
        """
        self.status = status
        self._content = content

    @property
    def content(self) -> Any:
        """
        模拟响应内容流

        Returns:
            Any: 异步内容生成器
        """
        outer_content = self._content

        class MockStreamReader:
            @staticmethod
            async def iter_chunked(size: int) -> AsyncGenerator[bytes, None]:
                yield outer_content

        return MockStreamReader()


@pytest.mark.asyncio
async def test_single_download(
    mock_downloader: ArxivDownloader,
    sample_paper: Paper,
    mocker: MockerFixture,
) -> None:
    mock_content = b"mock_pdf_content"
    mock_response = MockResponse(status=200, content=mock_content)
    request_mock = mocker.patch.object(
        mock_downloader.session_manager, "request", new_callable=mocker.AsyncMock
    )
    request_mock.return_value = mock_response

    filename = "test_paper.pdf"
    await mock_downloader.download_paper(sample_paper, filename)

    expected_path = mock_downloader.download_dir / filename
    assert expected_path.exists(), "Downloaded file should exist"

    content = expected_path.read_bytes()
    assert content == mock_content, "File content should match mock content"

    request_mock.assert_called_once_with("GET", str(sample_paper.pdf_url))



from datetime import datetime
import re
from time import monotonic
from types import SimpleNamespace
from typing import Optional
import xml.etree.ElementTree as ET
from zoneinfo import ZoneInfo

import aiohttp
from tenacity import RetryCallState

from aioarxiv.config import default_config
from aioarxiv.exception import ParseErrorContext, ParserException

from .log import logger


def create_trace_config() -> aiohttp.TraceConfig:
    """
    Create request tracing configuration.

    Returns:
        aiohttp.TraceConfig: Request tracing configuration object.
    """

    async def _on_request_start(
        session: aiohttp.ClientSession,
        trace_config_ctx: SimpleNamespace,
        params: aiohttp.TraceRequestStartParams,
    ) -> None:
        """
        Callback executed when a request starts.

        Args:
            session (aiohttp.ClientSession): The client session.
            trace_config_ctx (SimpleNamespace): Trace configuration context.
            params (aiohttp.TraceRequestStartParams): Request start parameters.
        """
        logger.debug(f"Starting request: {params.method} {params.url}")
        trace_config_ctx.start_time = monotonic()

    async def _on_request_end(
        session: aiohttp.ClientSession,
        trace_config_ctx: SimpleNamespace,
        params: aiohttp.TraceRequestEndParams,
    ) -> None:
        """
        Callback executed when a request ends.

        Args:
            session (aiohttp.ClientSession): The client session.
            trace_config_ctx (SimpleNamespace): Trace configuration context.
            params (aiohttp.TraceRequestEndParams): Request end parameters.
        """
        elapsed_time = monotonic() - trace_config_ctx.start_time
        logger.debug(
            f"Ending request: {params.response.status} {params.url} - Time elapsed: "
            f"{elapsed_time:.2f} seconds",
        )

    trace_config = aiohttp.TraceConfig()
    trace_config.on_request_start.append(_on_request_start)
    trace_config.on_request_end.append(_on_request_end)
    return trace_config


def create_parser_exception(
    data: ET.Element,
    url: Optional[str] = None,
    message: Optional[str] = None,
    namespace: Optional[str] = None,
    error: Optional[Exception] = None,
) -> ParserException:
    """
    Create a parsing exception for XML data parsing errors.

    Args:
        data (ET.Element): The data that failed to parse.
        url (Optional[str]): The request URL.
        message (Optional[str], optional): Exception message. Defaults to None.
        namespace (Optional[str], optional): XML namespace. Defaults to None.
        error (Optional[Exception], optional): Original exception. Defaults to None.

    Returns:
        ParserException: The created parsing exception.
    """
    return ParserException(
        url=url or "",
        message=message or "解析响应失败",
        context=ParseErrorContext(
            raw_content=ET.tostring(data, encoding="unicode"),
            element_name=data.tag,
            namespace=namespace,
        ),
        original_error=error,
    )


def calculate_page_size(
    config_page_size: int,
    start: int,
    max_results: Optional[int],
) -> int:
    """
    Calculate page size constrained by configuration page size and maximum results.

    Args:
        config_page_size (int): Configured page size.
        start (int): Starting position.
        max_results (Optional[int]): Maximum number of results.

    Returns:
        int: Calculated page size.
    """
    if max_results is None:
        return config_page_size

    return min(config_page_size, max_results - start)


def format_datetime(dt: datetime) -> str:
    """
    Format datetime to string.

    Args:
        dt (datetime): Datetime object to format.

    Returns:
        str: Formatted datetime string in format: %Y-%m-%d_%H-%M-%S_%Z
            (e.g., 2024-03-21_15-30-00_CST).

    Examples:
        >>> format_datetime(datetime(2024, 3, 21, 15, 30, 0))
        '2024-03-21_15-30-00_CST'
    """
    local_dt = dt.astimezone(ZoneInfo(default_config.timezone))
    return local_dt.strftime("%Y-%m-%d_%H-%M-%S_%Z")


def sanitize_title(title: str, max_length: int = 50) -> str:
    """
    Sanitize string to make it safe for use as a filename and limit its length.

    Sanitization rules:
    - Replace invalid characters with hyphens
    - Remove leading and trailing hyphens
    - Remove redundant hyphens
    - Truncate to max_length and append ellipsis if too long

    Args:
        title (str): The original title/filename to sanitize
        max_length (int, optional): Maximum length of the sanitized filename.
        Defaults to 50. If exceeded, the string will be truncated and '...' appended.

    Returns:
        str: The sanitized filename. If length exceeds max_length, returns truncated
        string with '...' appended.

    Examples:
        >>> sanitize_title("file/with*invalid:chars")
        'file-with-invalid-chars'
        >>> sanitize_title("very...long...title", max_length=10)
        'very-lo...'
    """
    # Strip whitespace and replace invalid chars with single hyphen
    sanitized = re.sub(r'[\\/*?:"<>|]+', "-", title.strip()).strip("-")

    # Truncate and append ellipsis if too long
    if len(sanitized) > max_length:
        sanitized = f"{sanitized[: max_length - 3].rstrip('-')}..."

    return sanitized


def log_retry_attempt(retry_state: RetryCallState) -> None:
    """
    Log retry attempt information.

    Args:
        retry_state (RetryCallState): Current retry state containing attempt
            information.

    Examples:
        >>> log_retry_attempt(RetryCallState(attempt_number=2))
        WARNING:root:retry times: 2/3
    """
    logger.warning(
        f"retry times: {retry_state.attempt_number}/{default_config.max_retries}"
    )



from datetime import datetime
from pathlib import Path
from typing import Any
from zoneinfo import ZoneInfo

from aiohttp import ClientResponse
from pydantic import AnyUrl, HttpUrl
import pytest
from pytest_mock import MockerFixture
from yarl import URL

from aioarxiv.client.arxiv_client import ArxivClient
from aioarxiv.client.downloader import ArxivDownloader
from aioarxiv.config import ArxivConfig
from aioarxiv.models import (
    Author,
    BasicInfo,
    Category,
    Metadata,
    Paper,
    PrimaryCategory,
    SearchParams,
    SearchResult,
)
from aioarxiv.utils.session import SessionManager

SAMPLE_XML_PATH = Path(__file__).parent.parent / "data" / "sample.xml"


@pytest.fixture
def sample_author() -> Author:
    """
    Create a sample author for testing

    Returns:
        Author: A sample author with name and affiliation
    """
    return Author(name="BalconyJH", affiliation="Test University")


@pytest.fixture
def sample_category() -> Category:
    """
    Create a sample category for testing

    Returns:
        Category: A sample category with primary and secondary classifications
    """
    return Category(
        primary=PrimaryCategory(
            term="cs.AI",
            scheme=AnyUrl("http://arxiv.org/schemas/atom"),
            label="Artificial Intelligence",
        ),
        secondary=["cs.LG", "stat.ML"],
    )


@pytest.fixture
def sample_basic_info(
    sample_author: Author, sample_category: Category, mock_datetime: datetime
) -> BasicInfo:
    """
    Create sample basic information for a paper

    Args:
        sample_author: The sample author fixture
        sample_category: The sample category fixture
        mock_datetime: The mocked datetime fixture

    Returns:
        BasicInfo: Basic information for a test paper
    """
    return BasicInfo(
        id="2312.12345",
        title="Test Paper Title",
        summary="Test paper summary",
        authors=[sample_author],
        categories=sample_category,
        published=mock_datetime,
        updated=mock_datetime,
    )


@pytest.fixture
def sample_paper(sample_basic_info: BasicInfo) -> Paper:
    """
    Create a sample paper for testing

    Args:
        sample_basic_info: The sample basic information fixture

    Returns:
        Paper: A complete paper object with all fields
    """
    return Paper(
        info=sample_basic_info,
        doi="10.1234/test.123",
        journal_ref="Test Journal Vol.1",
        pdf_url=HttpUrl("http://test.pdf"),
        comment="Test comment",
    )


@pytest.fixture
def sample_metadata(mock_datetime: datetime) -> Metadata:
    """
    Create sample metadata for testing

    Args:
        mock_datetime: The mocked datetime fixture

    Returns:
        Metadata: Metadata object with test values
    """
    return Metadata(
        start_time=mock_datetime,
        missing_results=0,
        pagesize=10,
        source=URL("http://export.arxiv.org/api/query"),
        end_time=None,
    )


@pytest.fixture
def sample_search_result(
    sample_paper: Paper, sample_metadata: Metadata
) -> SearchResult:
    """
    Create a sample search result for testing

    Args:
        sample_paper: Sample paper fixture
        sample_metadata: Sample metadata fixture

    Returns:
        SearchResult: Sample search result with test data
    """
    return SearchResult(
        papers=[sample_paper],
        total_result=1,
        page=1,
        has_next=False,
        query_params=SearchParams(
            query="test query", id_list=None, sort_by=None, sort_order=None
        ),
        metadata=sample_metadata,
    )


@pytest.fixture
def sample_arxiv_feed() -> str:
    """
    Load sample arXiv XML feed content from file

    Returns:
        str: Content of the sample XML feed file
    """
    return SAMPLE_XML_PATH.read_text(encoding="utf-8")


@pytest.fixture
def mock_session_manager(mocker: MockerFixture) -> Any:
    """
    Create a mocked session manager for testing

    Args:
        mocker: pytest mocker fixture

    Returns:
        Any: Mocked session manager with async methods
    """
    manager = mocker.Mock()
    manager.request = mocker.AsyncMock()
    manager.close = mocker.AsyncMock()
    return manager


@pytest.fixture
def mock_response(mocker: MockerFixture, sample_arxiv_feed: str) -> Any:
    """
    Create a mocked HTTP response for testing

    Args:
        mocker: pytest mocker fixture
        sample_arxiv_feed: Sample arXiv feed XML content

    Returns:
        Any: Mocked response object with status and content
    """
    response = mocker.Mock(spec=ClientResponse)
    response.status = 200
    response.text = mocker.AsyncMock(return_value=sample_arxiv_feed)
    response.url = "http://export.arxiv.org/api/query"
    return response


@pytest.fixture
def mock_datetime(mocker: MockerFixture) -> datetime:
    """
    Mock datetime.now() and return a fixed time (2025-01-02 13:37:50 UTC)

    Args:
        mocker: pytest mocker fixture

    Returns:
        datetime: Fixed datetime object for testing
    """
    fixed_dt = datetime(2025, 1, 2, 13, 37, 50, tzinfo=ZoneInfo("UTC"))
    datetime_mock = mocker.patch("datetime.datetime")
    datetime_mock.now.return_value = fixed_dt
    datetime_mock.now.side_effect = (
        lambda tz=None: fixed_dt.astimezone(tz) if tz else fixed_dt
    )
    return fixed_dt


@pytest.fixture
def mock_arxiv_client(mock_session_manager: Any) -> ArxivClient:
    """
    Create an ArXiv client instance for testing

    Args:
        mock_session_manager: Mocked session manager

    Returns:
        ArxivClient: Configured ArXiv client for testing
    """
    return ArxivClient(session_manager=mock_session_manager)


@pytest.fixture
def mock_downloader(
    download_dir: Path,
    mock_config: ArxivConfig,
    mocker: MockerFixture,
) -> ArxivDownloader:
    """
    Create an ArXiv downloader instance for testing

    Args:
        download_dir: Directory for downloads
        mock_config: ArXiv configuration
        mocker: pytest mocker fixture

    Returns:
        ArxivDownloader: Configured downloader instance
    """
    session_manager = mocker.Mock(spec=SessionManager)
    session_manager.request = mocker.AsyncMock()
    return ArxivDownloader(
        session_manager=session_manager,
        download_dir=download_dir,
        config=mock_config,
    )


@pytest.fixture
def mock_config() -> ArxivConfig:
    """
    Provide default arXiv configuration

    Returns:
        ArxivConfig: Default configuration for arXiv client
    """
    return ArxivConfig()


@pytest.fixture
def download_dir(tmp_path: Path) -> Path:
    """
    Create and provide a temporary download directory

    Args:
        tmp_path: pytest temporary path fixture

    Returns:
        Path: Path to the created downloads directory
    """
    download_path = tmp_path / "downloads"
    download_path.mkdir(parents=True)
    return download_path



from dataclasses import dataclass
from datetime import datetime
from enum import Enum
import re
from typing import Annotated, Optional
import uuid
from zoneinfo import ZoneInfo

from pydantic import (
    UUID4,
    AnyUrl,
    BaseModel,
    Field,
    HttpUrl,
    computed_field,
    field_validator,
)
from yarl import URL

from aioarxiv.config import default_config


class SortCriterion(str, Enum):
    """排序标准"""

    RELEVANCE = "relevance"
    LAST_UPDATED = "lastUpdatedDate"
    SUBMITTED = "submittedDate"


class SortOrder(str, Enum):
    """排序方向"""

    ASCENDING = "ascending"
    DESCENDING = "descending"


class Author(BaseModel):
    """作者模型"""

    name: str = Field(description="作者姓名")
    affiliation: Optional[str] = Field(None, description="作者的机构隶属信息")


class PrimaryCategory(BaseModel):
    """主分类模型"""

    term: str = Field(description="分类标识符")
    scheme: Optional[AnyUrl] = Field(None, description="分类系统的 URI")
    label: Optional[str] = Field(None, description="分类标签")


class Category(BaseModel):
    """分类模型"""

    primary: PrimaryCategory = Field(description="主分类")
    secondary: list[str] = Field(description="次级分类列表")


class BasicInfo(BaseModel):
    """基础论文信息"""

    id: str = Field(description="arXiv ID")
    title: str = Field(description="标题")
    summary: str = Field(description="摘要")
    authors: list[Author] = Field(description="作者列表")
    categories: Category = Field(description="分类")
    published: datetime = Field(description="发布时间")
    updated: datetime = Field(description="更新时间")


class Paper(BaseModel):
    """论文模型"""

    info: BasicInfo = Field(description="基础信息")
    doi: Optional[str] = Field(None, description="DOI, 格式需符合正则")
    journal_ref: Optional[str] = Field(None, description="期刊引用")
    pdf_url: Optional[HttpUrl] = Field(None, description="PDF下载链接")
    comment: Optional[str] = Field(None, description="作者评论或注释")

    @field_validator("doi")
    @classmethod
    def validate_doi(cls, v: Optional[str]) -> Optional[str]:
        if v is None:
            return v

        pattern = r"^10\.\d{4,9}/[-._;()/:a-zA-Z0-9]+$"
        if not re.match(pattern, v):
            msg = "Invalid DOI format. Must match pattern: 10.XXXX/suffix"
            raise ValueError(msg)
        return v


class SearchParams(BaseModel):
    """搜索参数"""

    query: str = Field(description="搜索关键词")
    id_list: Optional[list[str]] = Field(None, description="需要精确搜索的ID列表")
    start: Optional[int] = Field(default=0, ge=0, description="起始索引")
    max_results: Optional[int] = Field(default=10, gt=0, description="最大返回结果数")
    sort_by: Optional[SortCriterion] = Field(None, description="排序标准")
    sort_order: Optional[SortOrder] = Field(None, description="排序方向")


class Metadata(BaseModel):
    """元数据模型"""

    start_time: datetime = Field(
        default_factory=lambda: datetime.now(tz=ZoneInfo(default_config.timezone)),
        description="请求创建时间",
    )
    end_time: Optional[datetime] = Field(
        None,
        description="请求结束时间",
    )
    missing_results: int = Field(description="缺失结果数")
    pagesize: int = Field(description="每页结果数")
    source: URL = Field(description="数据源")

    model_config = {"arbitrary_types_allowed": True}

    @computed_field
    def duration_seconds(self) -> float:
        """持续时间(秒),  保留3位小数"""
        if self.end_time is None:
            return 0.000
        return round((self.end_time - self.start_time).total_seconds(), 3)

    @computed_field
    def duration_ms(self) -> float:
        """持续时间(毫秒),  保留3位小数"""
        if self.end_time is None:
            return 0.000
        delta = self.end_time - self.start_time
        return round(delta.total_seconds() * 1000, 3)


class SearchResult(BaseModel):
    """搜索结果模型"""

    id: UUID4 = Field(
        default_factory=lambda: uuid.uuid4(),
        description="结果ID",
    )
    papers: list[Paper] = Field(description="论文结果列表")
    total_result: int = Field(description="匹配的总论文数量")
    page: int = Field(description="当前页码")
    has_next: bool = Field(description="是否有下一页")
    query_params: SearchParams = Field(description="搜索参数")
    metadata: Metadata = Field(description="元数据")

    @computed_field
    def papers_count(self) -> int:
        return len(self.papers)


class DownloadStats(BaseModel):
    """下载统计数据"""

    total: int = Field(description="总下载数")
    completed: int = Field(default=0, description="完成数")
    failed: int = Field(default=0, description="失败数")
    start_time: datetime = Field(
        default_factory=lambda: datetime.now(tz=ZoneInfo(default_config.timezone)),
        description="开始时间",
    )
    end_time: Optional[datetime] = Field(default=None, description="结束时间")
    papers: Annotated[
        list[Paper], Field(default_factory=list, description="已下载论文")
    ]
    failed_papers: Annotated[
        list[tuple[Paper, Exception]],
        Field(default_factory=list, description="下载失败的论文"),
    ]

    model_config = {"arbitrary_types_allowed": True}


@dataclass
class PageParam:
    start: int
    end: int



import asyncio
from datetime import datetime
from io import StringIO
import math
from time import monotonic
from types import SimpleNamespace
from typing import cast
import xml.etree.ElementTree as ET
from zoneinfo import ZoneInfo

import aiohttp
from aiohttp import ClientResponse, TraceRequestEndParams, TraceRequestStartParams
from loguru import logger
from multidict import CIMultiDict
import pytest
from tenacity import RetryCallState
from yarl import URL

from aioarxiv.utils import (
    calculate_page_size,
    create_parser_exception,
    create_trace_config,
    format_datetime,
    log_retry_attempt,
    sanitize_title,
)

TOLERANCE = 0.25


@pytest.fixture
def sample_xml_element():
    """创建示例 XML 元素用于测试"""
    root = ET.Element("root")
    child = ET.SubElement(root, "child")
    child.text = "test content"
    return root


@pytest.fixture
def mock_retry_state():
    """创建模拟的 RetryCallState"""

    class MockRetryState:
        def __init__(self):
            self.attempt_number = 1

    return MockRetryState()


@pytest.fixture
def capture_debug_logs():
    """捕获 loguru 的 DEBUG 日志"""
    string_io = StringIO()

    handler_id = logger.add(
        string_io,
        format="{message}",
        level="DEBUG",
        catch=False,
    )

    yield string_io

    logger.remove(handler_id)


@pytest.fixture
def mock_session(mocker):
    """创建模拟的 aiohttp session"""
    return mocker.create_autospec(aiohttp.ClientSession)


@pytest.fixture
def mock_response(mocker):
    """创建模拟的 ClientResponse"""
    response = mocker.create_autospec(ClientResponse, instance=True)

    response.status = 200
    response.url = URL("http://test.com/api")
    response.read = b""
    response.close = mocker.AsyncMock()

    return response


@pytest.mark.asyncio
async def test_create_trace_config_request_lifecycle(
    mock_session, mock_response, capture_debug_logs
):
    """测试请求生命周期的追踪配置"""
    trace_config = create_trace_config()
    assert isinstance(trace_config, aiohttp.TraceConfig)

    ctx = SimpleNamespace()

    start_params = TraceRequestStartParams(
        method="GET", url=URL("http://test.com/api"), headers=CIMultiDict()
    )

    await trace_config.on_request_start[0](mock_session, ctx, start_params)

    start_log = capture_debug_logs.getvalue()
    assert "Starting request: GET http://test.com/api" in start_log

    start_time = ctx.start_time

    expected_delay = 0.1
    await asyncio.sleep(expected_delay)

    end_params = TraceRequestEndParams(
        method="GET",
        url=URL("http://test.com/api"),
        headers=CIMultiDict(),
        response=mock_response,
    )

    await trace_config.on_request_end[0](mock_session, ctx, end_params)

    full_log = capture_debug_logs.getvalue()

    assert "Ending request: 200 http://test.com/api" in full_log
    assert "Time elapsed:" in full_log
    assert "seconds" in full_log

    elapsed_time = monotonic() - start_time
    assert math.isclose(elapsed_time, expected_delay, rel_tol=TOLERANCE), (
        f"请求耗时不在预期范围内: 期望约为{expected_delay}秒, "
        f"实际为{elapsed_time:.4f}秒"
    )


@pytest.mark.asyncio
async def test_create_trace_config_error_case(
    mock_session, mock_response, mocker, capture_debug_logs
):
    """测试错误情况下的追踪配置"""
    trace_config = create_trace_config()
    ctx = SimpleNamespace()

    mocker.patch.object(mock_response, "status", 404)
    mocker.patch.object(mock_response, "url", URL("invalid://url"))

    start_params = TraceRequestStartParams(
        method="GET", url=URL("invalid://url"), headers=CIMultiDict()
    )

    await trace_config.on_request_start[0](mock_session, ctx, start_params)

    end_params = TraceRequestEndParams(
        method="GET",
        url=URL("invalid://url"),
        headers=CIMultiDict(),
        response=mock_response,
    )

    await trace_config.on_request_end[0](mock_session, ctx, end_params)

    log_output = capture_debug_logs.getvalue()

    assert "Starting request: GET invalid://url" in log_output
    assert "Ending request: 404 invalid://url" in log_output


@pytest.mark.asyncio
async def test_create_trace_config_multiple_requests(
    mock_session, mock_response, capture_debug_logs
):
    """
    测试多个连续请求的追踪配置
    """
    trace_config = create_trace_config()
    ctx = SimpleNamespace()

    async def make_request(delay: float):
        start_params = TraceRequestStartParams(
            method="GET", url=URL("http://test.com/api"), headers=CIMultiDict()
        )

        await trace_config.on_request_start[0](mock_session, ctx, start_params)
        start_time = ctx.start_time

        await asyncio.sleep(delay)

        end_params = TraceRequestEndParams(
            method="GET",
            url=URL("http://test.com/api"),
            headers=CIMultiDict(),
            response=mock_response,
        )

        await trace_config.on_request_end[0](mock_session, ctx, end_params)

        return monotonic() - start_time

    delays = [0.1, 0.2, 0.3]
    elapsed_times = []

    for delay in delays:
        elapsed_time = await make_request(delay)
        elapsed_times.append(elapsed_time)

    for expected, actual in zip(delays, elapsed_times):
        assert math.isclose(
            actual, expected, rel_tol=TOLERANCE
        ), f"请求耗时不在预期范围内: 期望约为{expected}秒, 实际为{actual:.4f}秒"


def test_create_parser_exception_basic(sample_xml_element):
    """
    测试创建基本的解析异常

    验证:
    1. 使用最基本的参数创建异常
    2. 检查默认消息
    """
    exception = create_parser_exception(data=sample_xml_element)

    assert exception.url == ""
    assert exception.message == "解析响应失败"
    assert exception.context is not None
    assert exception.context.element_name == "root"
    assert exception.context.raw_content is not None
    assert "<root><child>test content</child></root>" in exception.context.raw_content
    assert exception.context.namespace is None
    assert exception.original_error is None


def test_create_parser_exception_with_url(sample_xml_element):
    """
    测试创建带 URL 的解析异常

    验证:
    1. URL 被正确设置
    2. 其他参数保持默认值
    """
    test_url = "http://test.com/api"
    exception = create_parser_exception(data=sample_xml_element, url=test_url)

    assert exception.url == test_url
    assert exception.message == "解析响应失败"


def test_create_parser_exception_with_custom_message(sample_xml_element):
    """
    测试创建带自定义消息的解析异常

    验证:
    1. 自定义消息被正确设置
    2. 其他参数保持默认值
    """
    custom_message = "自定义错误消息"
    exception = create_parser_exception(data=sample_xml_element, message=custom_message)

    assert exception.url == ""
    assert exception.message == custom_message


def test_create_parser_exception_with_namespace(sample_xml_element):
    """
    测试创建带命名空间的解析异常

    验证:
    1. 命名空间被正确设置
    2. 其他参数保持默认值
    """
    namespace = "http://test.namespace"
    exception = create_parser_exception(data=sample_xml_element, namespace=namespace)

    assert exception.context is not None
    assert exception.context.namespace == namespace
    assert exception.url == ""
    assert exception.message == "解析响应失败"


def test_create_parser_exception_with_error(sample_xml_element):
    """
    测试创建带原始异常的解析异常

    验证:
    1. 原始异常被正确设置
    2. 其他参数保持默认值
    """
    original_error = ValueError("测试错误")
    exception = create_parser_exception(data=sample_xml_element, error=original_error)

    assert exception.original_error == original_error
    assert exception.url == ""
    assert exception.message == "解析响应失败"


def test_create_parser_exception_with_all_params(sample_xml_element):
    """
    测试创建包含所有参数的解析异常

    验证:
    1. 所有参数都被正确设置
    2. XML内容被正确转换为字符串
    """
    test_url = "http://test.com/api"
    custom_message = "自定义错误消息"
    namespace = "http://test.namespace"
    original_error = ValueError("测试错误")

    exception = create_parser_exception(
        data=sample_xml_element,
        url=test_url,
        message=custom_message,
        namespace=namespace,
        error=original_error,
    )

    assert exception.url == test_url
    assert exception.message == custom_message
    assert exception.context is not None
    assert exception.context.namespace == namespace
    assert exception.original_error == original_error
    assert exception.context.raw_content is not None
    assert "<root><child>test content</child></root>" in exception.context.raw_content
    assert exception.context.element_name == "root"


def test_create_parser_exception_with_complex_xml(sample_xml_element):
    """
    测试使用复杂XML创建解析异常

    验证:
    1. 复杂XML结构被正确处理
    2. 异常包含完整的XML内容
    """
    # 创建更复杂的XML结构
    sub_element = ET.SubElement(sample_xml_element, "nested")
    sub_element.set("attr", "value")
    sub_element.text = "nested content"

    exception = create_parser_exception(data=sample_xml_element)

    assert exception.context is not None
    assert exception.context.element_name == "root"
    assert exception.context.raw_content is not None
    assert 'attr="value"' in exception.context.raw_content
    assert "nested content" in exception.context.raw_content


# 测试 calculate_page_size
@pytest.mark.asyncio
async def test_calculate_page_size():
    test_cases = [
        (100, 0, None, 100),  # 无最大结果限制
        (100, 0, 50, 50),  # 最大结果小于页面大小
        (100, 30, 50, 20),  # 考虑起始位置
        (100, 0, 150, 100),  # 最大结果大于页面大小
    ]

    for config_size, start, max_results, expected in test_cases:
        result = calculate_page_size(config_size, start, max_results)
        assert result == expected


# 测试 format_datetime
@pytest.mark.asyncio
async def test_format_datetime():
    # 创建一个固定的UTC时间
    dt = datetime(2024, 12, 31, 22, 27, 42, tzinfo=ZoneInfo("UTC"))
    formatted = format_datetime(dt)

    # 验证格式
    assert isinstance(formatted, str)
    assert "_" in formatted
    assert formatted.endswith("CST")  # 假设默认时区为 CST


# 测试 sanitize_title
@pytest.mark.asyncio
async def test_sanitize_title():
    test_cases = [
        ("normal title", "normal title"),
        ("file/with*/invalid:chars", "file-with-invalid-chars"),
        ("a" * 100, "a" * 47 + "..."),  # 测试长度限制
        ("  spaces  ", "spaces"),  # 测试空格处理
        ('test"quote"test', "test-quote-test"),  # 测试引号处理
    ]

    for input_title, expected in test_cases:
        result = sanitize_title(input_title)
        assert result == expected


# 测试自定义长度限制
@pytest.mark.asyncio
async def test_sanitize_title_custom_length():
    result = sanitize_title("very long title", max_length=10)
    assert len(result) <= 10
    assert result.endswith("...")


@pytest.mark.asyncio
async def test_log_retry_attempt(mock_retry_state, capture_debug_logs):
    """测试重试日志记录功能"""
    # 执行日志记录
    log_retry_attempt(cast(RetryCallState, mock_retry_state))

    # 获取日志内容
    log_output = capture_debug_logs.getvalue()

    # 验证日志内容
    assert "retry times:" in log_output


# 测试边界情况
@pytest.mark.asyncio
async def test_edge_cases():
    # 测试空标题
    assert sanitize_title("") == ""

    # 测试全特殊字符的标题
    assert sanitize_title("***:::///") == ""

    # 测试计算页面大小的边界情况
    assert calculate_page_size(100, 999, 1000) == 1
    assert calculate_page_size(100, 1000, 1000) == 0


# 测试异常情况
@pytest.mark.asyncio
async def test_exception_cases(sample_xml_element):
    # 测试无参数创建解析异常
    exception = create_parser_exception(sample_xml_element)
    assert exception.url == ""
    assert exception.message == "解析响应失败"



from typing import Optional

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict


class ArxivConfig(BaseSettings):
    """arXiv API 配置类"""

    base_url: str = Field(
        default="http://export.arxiv.org/api/query",
        description="arXiv API 基础URL",
    )
    timeout: float = Field(default=30.0, description="请求超时时间(秒)", gt=0)
    timezone: str = Field(default="Asia/Shanghai", description="时区")
    max_retries: int = Field(default=3, description="最大重试次数", ge=0)
    rate_limit_calls: int = Field(
        default=1,
        description="速率限制窗口内的最大请求数",
        ge=0,
    )
    rate_limit_period: float = Field(
        default=3.0,
        description="速率限制窗口期(秒)",
        ge=0,
    )
    max_concurrent_requests: int = Field(default=1, description="最大并发请求数")
    proxy: Optional[str] = Field(default=None, description="HTTP/HTTPS代理URL")
    log_level: str = Field(default="INFO", description="日志等级")
    page_size: int = Field(default=1000, description="每页结果数")
    min_wait: float = Field(default=3.0, description="最小重试等待时间(秒)", gt=0)

    model_config = SettingsConfigDict(
        env_prefix="ARXIV_",
        env_file_encoding="utf-8",
        case_sensitive=False,
        env_file=".env",
        extra="allow",
    )


default_config = ArxivConfig()



import asyncio
from datetime import datetime
from pathlib import Path
from types import TracebackType
from typing import Optional
from zoneinfo import ZoneInfo

from aiohttp import ClientResponse

from aioarxiv.config import ArxivConfig, default_config
from aioarxiv.exception import HTTPException, QueryBuildError, QueryContext
from aioarxiv.models import (
    Metadata,
    PageParam,
    Paper,
    SearchParams,
    SearchResult,
    SortCriterion,
    SortOrder,
)
from aioarxiv.utils import logger
from aioarxiv.utils.log import set_config
from aioarxiv.utils.parser import ArxivParser
from aioarxiv.utils.session import SessionManager

from .downloader import ArxivDownloader, DownloadTracker


class ArxivClient:
    def __init__(
        self,
        config: Optional[ArxivConfig] = None,
        session_manager: Optional[SessionManager] = None,
        enable_downloader: bool = False,
        download_dir: Optional[Path] = None,
    ) -> None:
        """Initialize ArxivClient with optional configuration.

        Args:
            config (Optional[ArxivConfig]): Custom configuration for the client.
            session_manager (Optional[SessionManager]): Custom session manager.
            enable_downloader (bool): Whether to enable the paper downloader.
            download_dir (Optional[Path]): Directory path for downloading papers.
        """
        self._config = config or default_config
        self._session_manager = session_manager or SessionManager(config=self._config)
        self.download_dir = download_dir
        self._enable_downloader = enable_downloader
        self._downloader: Optional[ArxivDownloader] = None
        set_config(self._config)
        logger.info(f"ArxivClient initialized with config: {self._config.model_dump()}")

    @property
    def downloader(self) -> Optional[ArxivDownloader]:
        """Get the downloader instance if enabled."""
        if not self._enable_downloader:
            logger.debug("Downloader is disabled")
            return None
        if self._downloader is None:
            self._downloader = ArxivDownloader(
                self._session_manager,
                self.download_dir,
                self._config,
            )
        return self._downloader

    def _build_search_result_metadata(
        self,
        searchresult: SearchResult,
        page: int,
        batch_size: int,
        papers: list[Paper],
    ) -> SearchResult:
        """Build search result metadata with updated information.

        Args:
            searchresult (SearchResult): Search result object.
            page (int): Page number of the search result.
            batch_size (int): Number of papers fetched in the batch.
            papers (list[Paper]): List of papers fetched in the batch.

        Returns:
            SearchResult: Search result object with updated metadata.
        """
        has_next = searchresult.total_result > (page * batch_size)
        metadata = searchresult.metadata.model_copy(
            update={
                "end_time": datetime.now(tz=ZoneInfo(default_config.timezone)),
                "pagesize": self._config.page_size,
            },
        )
        return searchresult.model_copy(
            update={
                "papers": papers,
                "page": page,
                "has_next": has_next,
                "metadata": metadata,
            },
        )

    async def _prepare_initial_search(
        self,
        query: str,
        start: Optional[int],
        id_list: Optional[list[str]],
        max_results: Optional[int],
        sort_by: Optional[SortCriterion],
        sort_order: Optional[SortOrder],
    ) -> tuple[SearchResult, bool]:
        """Prepare the initial search request and fetch the first page of results.

        Args:
            query (str): The search query string.
            start (Optional[int]): Index of first result to retrieve.
            id_list (Optional[list[str]]): List of arXiv IDs to retrieve.
            max_results (Optional[int]): Maximum number of results to return.
            sort_by (Optional[SortCriterion]): Criterion to sort results by.
            sort_order (Optional[SortOrder]): Order of sorting.

        Returns:
            tuple[SearchResult, bool]: Tuple containing search result and flag
                indicating whether more results need to be fetched.
        """
        page_size = min(self._config.page_size, max_results or self._config.page_size)

        params = SearchParams(
            query=query,
            start=start,
            id_list=id_list,
            max_results=page_size,
            sort_by=sort_by,
            sort_order=sort_order,
        )

        response = await self._fetch_page(params)
        result = ArxivParser(await response.text(), response).build_search_result(
            params
        )
        logger.debug(f"Fetched page 1 with {len(result.papers)} papers")
        result = self._build_search_result_metadata(
            searchresult=result, page=1, batch_size=page_size, papers=result.papers
        )

        needs_more = (
            max_results is not None
            and max_results > len(result.papers)
            and result.total_result > len(result.papers)
        )

        return result, needs_more

    async def _fetch_and_update_result(
        self, params: SearchParams, page: int
    ) -> SearchResult:
        """Fetch a single page of search results and update metadata.

        Args:
            params (SearchParams): Search parameters for the API request.
            page (int): Page number of the search result.

        Returns:
            SearchResult: Search result with updated metadata.
        """
        response = await self._fetch_page(params)
        result = ArxivParser(await response.text(), response).build_search_result(
            params
        )
        return self._build_search_result_metadata(
            searchresult=result,
            page=page,
            batch_size=self._config.page_size,
            papers=result.papers,
        )

    async def _create_batch_tasks(
        self,
        query: str,
        page_params: list[PageParam],
        sort_by: Optional[SortCriterion] = None,
        sort_order: Optional[SortOrder] = None,
    ) -> list[asyncio.Task[SearchResult]]:
        """Create a list of tasks to fetch multiple pages of search results.

        Args:
            query (str): The search query string.
            page_params (list[PageParam]): List of page parameters for batch requests.
            sort_by (Optional[SortCriterion]): Criterion to sort results by.
            sort_order (Optional[SortOrder]): Order of sorting.

        Returns:
            list[asyncio.Task[SearchResult]]: List of tasks to fetch search results.
        """
        return [
            asyncio.create_task(
                self._fetch_and_update_result(
                    SearchParams(
                        query=query,
                        start=param.start,
                        max_results=param.end - param.start,
                        sort_by=sort_by,
                        sort_order=sort_order,
                        id_list=None,
                    ),
                    page=i + 2,
                )
            )
            for i, param in enumerate(page_params)
        ]

    async def search(
        self,
        query: str,
        id_list: Optional[list[str]] = None,
        max_results: Optional[int] = None,
        sort_by: Optional[SortCriterion] = None,
        sort_order: Optional[SortOrder] = None,
        start: Optional[int] = None,
    ) -> SearchResult:
        """Search arXiv papers based on given parameters.

        Args:
            query (str): The search query string.
            id_list (Optional[list[str]], optional): List of arXiv IDs to retrieve.
                Defaults to None.
            max_results (Optional[int], optional): Maximum number of results to return.
                Defaults to None (uses config default).
            sort_by (Optional[SortCriterion], optional): Criterion to sort results by.
                Defaults to None.
            sort_order (Optional[SortOrder], optional): Order of sorting. Defaults to
                None.
            start (Optional[int], optional): Index of first result to retrieve.
                Defaults to None (starts from beginning).

        Returns:
            SearchResult: Object containing search results and metadata.

        Raises:
            HTTPException: If the API request fails.
            QueryBuildError: If there's an error building the search query.
        """
        try:
            # Get initial search results
            first_page_result, should_fetch_more = await self._prepare_initial_search(
                query=query,
                start=start,
                id_list=id_list,
                max_results=max_results,
                sort_by=sort_by,
                sort_order=sort_order,
            )

            if not should_fetch_more:
                return first_page_result

            # Calculate pagination parameters
            papers_received = len(first_page_result.papers)
            remaining_papers = min(
                (max_results - papers_received)
                if max_results
                else first_page_result.total_result,
                first_page_result.total_result - papers_received,
            )

            if remaining_papers <= 0:
                return first_page_result

            # Prepare batch requests parameters
            page_params = self._generate_page_params(
                base_start=(start or 0) + self._config.page_size,
                remaining_papers=remaining_papers,
                page_size=self._config.page_size,
            )

            logger.debug(f"Fetching {len(page_params)} additional pages")

            # Execute batch requests
            additional_results = await self._fetch_batch_results(
                query=query,
                page_params=page_params,
                sort_by=sort_by,
                sort_order=sort_order,
            )

            return self.aggregate_search_results(
                [first_page_result, *additional_results]
            )

        except Exception as e:
            logger.error(f"Search operation failed: {e!s}", exc_info=True)
            raise

    async def _fetch_page(self, params: SearchParams) -> ClientResponse:
        """Fetch a single page of results from arXiv API.

        Args:
            params (SearchParams): Search parameters for the API request.

        Returns:
            ClientResponse: HTTP response from the arXiv API.

        Raises:
            HTTPException: If the API request returns a non-200 status code.
        """
        query_params = self._build_query_params(params)
        response = await self._session_manager.request(
            "GET", str(self._config.base_url), params=query_params
        )

        if response.status != 200:
            raise HTTPException(response.status)

        return response

    @staticmethod
    def _generate_page_params(
        base_start: int, remaining_papers: int, page_size: int
    ) -> list[PageParam]:
        """
        Generate page parameters for batch requests.

        Args:
            base_start (int): Starting index for the first page.
            remaining_papers (int): Number of papers remaining to fetch.
            page_size (int): Number of papers to fetch per page.

        Returns:
            list[PageParam]: List of page parameters for batch requests.
        """
        total_pages = (remaining_papers + page_size - 1) // page_size
        page_params = []

        for page in range(total_pages):
            page_start = base_start + page * page_size
            page_end = min(page_start + page_size, base_start + remaining_papers)
            if page_end > page_start:
                page_params.append(PageParam(start=page_start, end=page_end))

        return page_params

    async def _fetch_batch_results(
        self,
        query: str,
        page_params: list[PageParam],
        sort_by: Optional[SortCriterion],
        sort_order: Optional[SortOrder],
    ) -> list[SearchResult]:
        """
        Fetch multiple pages of results from arXiv API.

        Args:
            query (str): The search query string.
            page_params (list[PageParam]): List of page parameters for batch requests.
            sort_by (Optional[SortCriterion]): Criterion to sort results by.
            sort_order (Optional[SortOrder]): Order of sorting.

        Returns:
            list[SearchResult]: List of search results from batch requests.
        """
        tasks = await self._create_batch_tasks(query, page_params, sort_by, sort_order)

        if not tasks:
            return []

        responses = await asyncio.gather(*tasks, return_exceptions=True)
        valid_results = []

        for response in responses:
            if isinstance(response, SearchResult):
                valid_results.append(response)
            elif isinstance(response, Exception):
                logger.error(f"Batch task failed: {response!s}", exc_info=True)

        return valid_results

    def _build_query_params(self, params: SearchParams) -> dict:
        """
        Build query parameters for arXiv API request.

        Args:
            params (SearchParams): Search parameters for the API request.

        Returns:
            dict: Query parameters for the API request.

        Raises:
            QueryBuildError: If there's an error building the search query.
        """
        try:
            query_params = {
                "search_query": params.query,
                "start": params.start or 0,
                "max_results": params.max_results,
            }

            if params.id_list:
                query_params["id_list"] = ",".join(params.id_list)

            if params.sort_by:
                query_params["sortBy"] = params.sort_by.value

            if params.sort_order:
                query_params["sortOrder"] = params.sort_order.value

            return query_params

        except Exception as e:
            raise QueryBuildError(
                message="Search query build failed",
                context=QueryContext(
                    params={
                        "page_size": self._config.page_size,
                        "max_results": params.max_results,
                        "start": params.start,
                        "id_list": params.id_list,
                    },
                    field_name="query_params",
                ),
                original_error=e,
            ) from e

    async def download_paper(
        self,
        paper: Paper,
        filename: Optional[str] = None,
    ) -> Optional[None]:
        """Download a single paper from arXiv.

        Args:
            paper (Paper): Paper object containing download information.
            filename (Optional[str], optional): Custom filename for the downloaded
                paper. Defaults to None.

        Returns:
            Optional[None]: None if downloader is disabled.

        Raises:
            PaperDownloadException: If paper download fails.
        """
        if downloader := self.downloader:
            await downloader.download_paper(paper, filename)
        return None

    async def download_search_result(
        self,
        search_result: SearchResult,
    ) -> Optional[DownloadTracker]:
        """Download all papers from a search result.

        Args:
            search_result (SearchResult): Search result containing papers to download.

        Returns:
            Optional[DownloadTracker]: Download tracker if downloader is enabled,
                None otherwise.
        """
        if downloader := self.downloader:
            return await downloader.batch_download(search_result)
        return None

    @staticmethod
    def _merge_paper_lists(
        papers_lists: list[list[Paper]], *, keep_latest: bool = True
    ) -> list[Paper]:
        """
        Merge multiple lists of papers into a single list.

        Args:
            papers_lists (list[list[Paper]]): List of lists of papers to merge.
            keep_latest (bool): Whether to keep the latest version of each paper.

        Returns:
            list[Paper]: List of unique papers.

        Raises:
            ValueError: If papers_lists is empty.
        """
        unique_papers: dict[str, Paper] = {}

        for papers in papers_lists:
            for paper in papers:
                paper_id = paper.info.id
                if paper_id not in unique_papers or (
                    keep_latest
                    and paper.info.updated > unique_papers[paper_id].info.updated
                ):
                    unique_papers[paper_id] = paper

        return list(unique_papers.values())

    def aggregate_search_results(self, results: list[SearchResult]) -> SearchResult:
        """Aggregate multiple search results into a single result.

        Args:
            results (list[SearchResult]): List of search results to aggregate.

        Returns:
            SearchResult: Combined search result with merged papers and metadata.

        Raises:
            ValueError: If results list is empty.
        """
        if not results:
            raise ValueError("Results list cannot be empty")

        papers_lists = [result.papers for result in results]
        merged_papers = self._merge_paper_lists(papers_lists)

        base_result = results[0]
        base_timezone = base_result.metadata.start_time.tzinfo

        aggregated_metadata = Metadata(
            start_time=min(
                result.metadata.start_time.astimezone(base_timezone)
                for result in results
            ),
            end_time=max(
                (
                    result.metadata.end_time.astimezone(base_timezone)
                    for result in results
                    if result.metadata.end_time is not None
                ),
                default=None,
            ),
            missing_results=sum(result.metadata.missing_results for result in results),
            pagesize=sum(result.metadata.pagesize for result in results),
            source=base_result.metadata.source,
        )

        aggregated_params = base_result.query_params.model_copy(
            update={
                "max_results": len(merged_papers),
                "start": min(result.query_params.start or 0 for result in results),
            }
        )

        aggregated_result = SearchResult(
            papers=merged_papers,
            total_result=max(result.total_result for result in results),
            page=max(result.page for result in results),
            has_next=any(result.has_next for result in results),
            query_params=aggregated_params,
            metadata=aggregated_metadata,
        )

        logger.debug(
            f"Aggregated {len(results)} search results with {len(merged_papers)} "
            f"papers in {aggregated_result.metadata.duration_seconds} seconds"
        )

        return aggregated_result

    async def close(self) -> None:
        """Close the client and cleanup resources."""
        await self._session_manager.close()

    async def __aenter__(self) -> "ArxivClient":
        """Enter the async context manager.

        Returns:
            ArxivClient: The client instance.
        """
        return self

    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        """Exit the async context manager and cleanup resources.

        Args:
            exc_type: Exception type if an exception occurred.
            exc_val: Exception value if an exception occurred.
            exc_tb: Exception traceback if an exception occurred.
        """
        await self.close()



from dataclasses import dataclass
from http import HTTPStatus
from typing import Any, Optional

from pydantic import BaseModel, HttpUrl


class ArxivException(Exception):
    """基础异常类"""

    def __str__(self) -> str:
        return super().__repr__()


class HTTPException(ArxivException):
    """HTTP 请求相关异常"""

    def __init__(self, status_code: int, message: Optional[str] = None) -> None:
        self.status_code = status_code
        self.message = message or HTTPStatus(status_code).description
        super().__init__(self.message)


class RateLimitException(HTTPException):
    """达到 API 速率限制时的异常"""

    def __init__(self, retry_after: Optional[int] = None) -> None:
        self.retry_after = retry_after
        super().__init__(429, "Too Many Requests")


class ValidationException(ArxivException):
    """数据验证异常"""

    def __init__(
        self,
        message: str,
        field_name: str,
        input_value: Any,
        expected_type: type,
        model: Optional[type[BaseModel]] = None,
        validation_errors: Optional[dict] = None,
    ) -> None:
        self.field_name = field_name
        self.input_value = input_value
        self.expected_type = expected_type
        self.model = model
        self.validation_errors = validation_errors
        super().__init__(message)

    def __str__(self) -> str:
        error_msg = [
            f"Validation error for field '{self.field_name}':",
            f"Input value: {self.input_value!r}",
            f"Expected type: {self.expected_type.__name__}",
        ]

        if self.model:
            error_msg.append(f"Model: {self.model.__name__}")

        if self.validation_errors:
            error_msg.append("Detailed errors:")
            error_msg.extend(
                f"  - {key}: {err}" for key, err in self.validation_errors.items()
            )
        return "\n".join(error_msg)


class TimeoutException(ArxivException):
    """请求超时异常"""

    def __init__(
        self,
        timeout: float,
        message: Optional[str] = None,
        proxy: Optional[HttpUrl] = None,
        link: Optional[HttpUrl] = None,
    ) -> None:
        self.timeout = timeout
        self.proxy = proxy
        self.link = link
        self.message = message or f"Request timed out after {timeout} seconds"
        super().__init__(message)

    def __str__(self) -> str:
        error_msg = [
            f"Request timed out after {self.timeout} seconds",
            self.message,
        ]

        if self.proxy:
            error_msg.append(f"Proxy: {self.proxy}")

        if self.link:
            error_msg.append(f"Link: {self.link}")

        return "\n".join(error_msg)


@dataclass
class ConfigError:
    """配置错误详情"""

    property_name: str
    input_value: Any
    expected_type: type
    message: str


class ConfigurationError(ArxivException):
    """配置错误异常"""

    def __init__(
        self,
        message: str,
        property_name: str,
        input_value: Any,
        expected_type: type,
        config_class: Optional[type] = None,
    ) -> None:
        self.property_name = property_name
        self.input_value = input_value
        self.expected_type = expected_type
        self.config_class = config_class
        self.message = message
        super().__init__(message)

    def __str__(self) -> str:
        error_parts = [
            f"Configuration error for '{self.property_name}':",
            f"Input value: {self.input_value!r}",
            f"Expected type: {self.expected_type.__name__}",
            f"Message: {self.message}",
        ]

        if self.config_class:
            error_parts.append(f"Config class: {self.config_class.__name__}")

        return "\n".join(error_parts)


@dataclass
class QueryContext:
    """查询构建上下文"""

    params: dict[str, Any]  # 查询参数
    field_name: Optional[str] = None  # 出错的字段
    value: Optional[Any] = None  # 问题值
    constraint: Optional[str] = None  # 违反的约束


class QueryBuildError(ArxivException):
    """查询构建错误"""

    def __init__(
        self,
        message: str,
        context: Optional[QueryContext] = None,
        original_error: Optional[Exception] = None,
    ) -> None:
        self.message = message
        self.context = context
        self.original_error = original_error
        super().__init__(message)

    def __str__(self) -> str:
        error_parts = [f"查询构建错误: {self.message}"]

        if self.context:
            if self.context.params:
                error_parts.append("参数:")
                error_parts.extend(
                    f"  • {k}: {v!r}" for k, v in self.context.params.items()
                )

            if self.context.field_name:
                error_parts.append(f"问题字段: {self.context.field_name}")

            if self.context.value is not None:
                error_parts.append(f"问题值: {self.context.value!r}")

            if self.context.constraint:
                error_parts.append(f"约束条件: {self.context.constraint}")

        if self.original_error:
            error_parts.extend(
                [
                    f"原始错误: {self.original_error!s}",
                    f"原始错误类型: {type(self.original_error).__name__}",
                ],
            )

        return "\n".join(error_parts)


@dataclass
class ParseErrorContext:
    """解析错误上下文"""

    raw_content: Optional[str] = None
    position: Optional[int] = None
    element_name: Optional[str] = None
    namespace: Optional[str] = None


class ParserException(Exception):
    """XML解析异常"""

    def __init__(
        self,
        url: str,
        message: str,
        context: Optional[ParseErrorContext] = None,
        original_error: Optional[Exception] = None,
    ) -> None:
        self.url = url
        self.message = message
        self.context = context
        self.original_error = original_error
        super().__init__(self.message)

    def __str__(self) -> str:
        parts = [f"解析错误: {self.message}", f"URL: {self.url}"]

        if self.context:
            if self.context.element_name:
                parts.append(f"元素: {self.context.element_name}")
            if self.context.namespace:
                parts.append(f"命名空间: {self.context.namespace}")
            if self.context.position is not None:
                parts.append(f"位置: {self.context.position}")
            if self.context.raw_content:
                parts.append(f"原始内容: \n{self.context.raw_content[:200]}...")

        if self.original_error:
            parts.append(f"原始错误: {self.original_error!s}")

        return "\n".join(parts)


class SearchCompleteException(ArxivException):
    """搜索完成异常"""

    def __init__(self, total_results: int) -> None:
        self.total_results = total_results
        super().__init__(f"搜索完成,共获取{total_results}条结果")


class PaperDownloadException(ArxivException):
    """论文下载异常"""

    def __init__(self, message: str) -> None:
        self.message = message
        super().__init__(message)

    def __str__(self) -> str:
        return f"论文下载异常: {self.message}"



from datetime import datetime
import pathlib
from typing import cast
import xml.etree.ElementTree as ET
from zoneinfo import ZoneInfo

from aiohttp import ClientResponse
from pydantic import HttpUrl
import pytest
from yarl import URL

from aioarxiv.exception import ParserException
from aioarxiv.models import Category, Paper
from aioarxiv.utils.parser import ArxivParser, PaperParser

SAMPLE_XML_PATH = pathlib.Path(__file__).parent.parent / "data" / "sample.xml"


@pytest.fixture
def sample_xml():
    return SAMPLE_XML_PATH.read_text(encoding="utf-8")


@pytest.fixture
def mock_response(mocker, sample_xml):
    response = mocker.AsyncMock(spec=ClientResponse)
    response.url = URL("http://test.com")
    response.text = mocker.AsyncMock(return_value=sample_xml)
    return response


@pytest.fixture
def paper_entry(sample_xml):
    root = ET.fromstring(sample_xml)
    return root.find("{http://www.w3.org/2005/Atom}entry")


def test_paper_parser_init(paper_entry):
    parser = PaperParser(paper_entry)
    assert parser.entry == paper_entry


def test_parse_authors(paper_entry):
    parser = PaperParser(paper_entry)
    authors = parser.parse_authors()
    assert len(authors) == 5
    assert authors[0].name == "David Prendergast"
    assert authors[0].affiliation == "Department of Physics"


def test_parse_categories(paper_entry):
    parser = PaperParser(paper_entry)
    categories = parser.parse_categories()
    assert isinstance(categories, Category)
    assert categories.primary.term == "cond-mat.str-el"
    assert categories.primary.label is None
    assert len(categories.secondary) == 0


def test_parse_basic_info(paper_entry):
    parser = PaperParser(paper_entry)
    info = parser.parse_basics_info()
    assert info.id == "0102536v1"
    assert (
        info.title
        == "Impact of Electron-Electron Cusp on Configuration Interaction Energies"
    )
    assert (
        info.summary
        == """  The effect of the electron-electron cusp on the convergence of configuration
            interaction (CI) wave functions is examined. By analogy with the
            pseudopotential approach for electron-ion interactions, an effective
            electron-electron interaction is developed which closely reproduces the
            scattering of the Coulomb interaction but is smooth and finite at zero
            electron-electron separation. The exact many-electron wave function for this
            smooth effective interaction has no cusp at zero electron-electron separation.
            We perform CI and quantum Monte Carlo calculations for He and Be atoms, both
            with the Coulomb electron-electron interaction and with the smooth effective
            electron-electron interaction. We find that convergence of the CI expansion of
            the wave function for the smooth electron-electron interaction is not
            significantly improved compared with that for the divergent Coulomb interaction
            for energy differences on the order of 1 mHartree. This shows that, contrary to
            popular belief, description of the electron-electron cusp is not a limiting
            factor, to within chemical accuracy, for CI calculations.
        """  # noqa: E501
    )
    assert len(info.authors) == 5
    assert isinstance(info.categories, Category)
    assert isinstance(info.published, datetime)
    assert isinstance(info.updated, datetime)


def test_parse_pdf_url(paper_entry):
    parser = PaperParser(paper_entry)
    url = parser.parse_pdf_url()
    assert url == cast(HttpUrl, "http://arxiv.org/pdf/cond-mat/0102536v1")


def test_parse_optional_fields(paper_entry):
    parser = PaperParser(paper_entry)
    fields = parser.parse_optional_fields()
    assert fields["doi"] == "10.1063/1.1383585"
    assert fields["comment"] == (
        """11 pages, 6 figures, 3 tables, LaTeX209, submitted to The Journal of
            Chemical Physics"""
    )
    assert fields["journal_ref"] == "J. Chem. Phys. 115, 1626 (2001)"


def test_parse_datetime():
    parser = PaperParser(ET.Element("entry"))
    dt = parser.parse_datetime("2024-03-18T00:00:00Z")
    assert isinstance(dt, datetime)
    assert dt.tzinfo == ZoneInfo("Asia/Shanghai")


def test_arxiv_parser_build_paper(paper_entry):
    paper = ArxivParser.build_paper(paper_entry)
    assert isinstance(paper, Paper)
    assert paper.info.id == "0102536v1"
    assert str(paper.pdf_url) == "http://arxiv.org/pdf/cond-mat/0102536v1"


@pytest.mark.asyncio
async def test_arxiv_parser_parse_feed(mock_response, sample_xml):
    parser = ArxivParser(sample_xml, mock_response)
    papers = parser.parse_feed()
    assert len(papers) == 1
    assert isinstance(papers[0], Paper)


def test_error_handling_missing_author(paper_entry):
    # Remove all authors
    for author in paper_entry.findall("{http://www.w3.org/2005/Atom}author"):
        paper_entry.remove(author)

    parser = PaperParser(paper_entry)
    with pytest.raises(ParserException):
        parser.parse_authors()


def test_error_handling_invalid_date():
    parser = PaperParser(ET.Element("entry"))
    with pytest.raises(ValueError, match=r"日期格式: .* 不符合预期"):
        parser.parse_datetime("invalid-date")


def test_error_handling_missing_pdf_url(paper_entry):
    # Remove PDF link
    for link in paper_entry.findall("{http://www.w3.org/2005/Atom}link"):
        paper_entry.remove(link)

    parser = PaperParser(paper_entry)
    assert parser.parse_pdf_url() is None



from types import TracebackType
from typing import Optional
from typing_extensions import Self

from aiohttp import ClientResponse, ClientSession, ClientTimeout, TraceConfig

from aioarxiv.config import ArxivConfig, default_config
from aioarxiv.utils import create_trace_config

from .log import logger
from .rate_limiter import RateLimiter


class SessionManager:
    """A session manager that handles HTTP requests with rate limiting and connection
    management.

    This class provides a convenient way to manage HTTP sessions with built-in rate
    limiting, timeout configuration, and proxy support. It ensures proper resource
    cleanup and provides an async context manager interface.

    Args:
        config (Optional[ArxivConfig]): Configuration for arXiv API and rate limiting.
            Defaults to default_config.
        session (Optional[ClientSession]): An existing aiohttp session to use.
            Defaults to None.
        trace_config (Optional[TraceConfig]): Configuration for request tracing.
            Defaults to None.

    Usage:
        ```python
        # Basic usage with default configuration
        async with SessionManager() as session:
            response = await session.request('GET', 'http://api.example.com/data')
            data = await response.json()

        # Custom configuration with rate limiting
        config = ArxivConfig(
            rate_limit_calls=5,
            rate_limit_period=1.0,
            timeout=30.0,
            proxy='http://proxy.example.com'
        )

        async with SessionManager(config=config) as session:
            # Makes rate-limited requests
            responses = await asyncio.gather(*(
                session.request('GET', f'http://api.example.com/item/{i}')
                for i in range(10)
            ))

        # Error handling
        async with SessionManager() as session:
            try:
                response = await session.request('GET', 'http://api.example.com/data')
                if response.status == 429:  # Too Many Requests
                    logger.warning("Rate limit exceeded")
            except Exception as e:
                logger.error(f"Request failed: {e}")
        ```
    """

    def __init__(
        self,
        config: Optional[ArxivConfig] = None,
        session: Optional[ClientSession] = None,
        trace_config: Optional[TraceConfig] = None,
    ) -> None:
        """Initialize the session manager.

        Args:
            config (Optional[ArxivConfig]): arXiv API configuration object.
                Defaults to default_config.
            session (Optional[ClientSession]): Existing aiohttp session to use.
                Defaults to None.
            trace_config (Optional[TraceConfig]): Request tracing configuration.
                Defaults to None.
        """
        self._config = config or default_config
        self._timeout = ClientTimeout(total=self._config.timeout)
        self._session = session
        self._trace_config = trace_config or create_trace_config()
        self._rate_limiter = RateLimiter(
            calls=self._config.rate_limit_calls,
            period=self._config.rate_limit_period,
        )

    @property
    async def session(self) -> ClientSession:
        """Get or create an aiohttp session.

        Returns:
            ClientSession: The active aiohttp session.

        Note:
            Creates a new session if one doesn't exist or if the existing session is
            closed.
        """
        if self._session is None or self._session.closed:
            self._session = ClientSession(
                timeout=self._timeout,
                trace_configs=[self._trace_config],
            )
        return self._session

    @property
    def rate_limiter(self) -> RateLimiter:
        """Get the rate limiter instance.

        Returns:
            RateLimiter: The rate limiter configured for this session.
        """
        return self._rate_limiter

    async def request(self, method: str, url: str, **kwargs) -> ClientResponse:
        """Send a rate-limited HTTP request.

        Args:
            method (str): HTTP method to use (e.g., 'GET', 'POST').
            url (str): Target URL for the request.
            **kwargs: Additional arguments to pass to session.request.

        Returns:
            ClientResponse: The aiohttp response object.

        Raises:
            aiohttp.ClientError: If the request fails.
        """

        @self._rate_limiter.limit()
        async def _rate_limited_request() -> ClientResponse:
            if self._config.proxy:
                logger.debug(
                    "Using proxy for request",
                    extra={"proxy": self._config.proxy, "url": url},
                )
                kwargs["proxy"] = self._config.proxy

            client = await self.session
            return await client.request(method, url, **kwargs)

        return await _rate_limited_request()

    async def close(self) -> None:
        """Close the session and cleanup resources.

        This method ensures that the underlying aiohttp session is properly closed
        and resources are released.
        """
        if self._session and not self._session.closed:
            await self._session.close()
            self._session = None

    async def __aenter__(self) -> Self:
        """Enter the session manager context.

        Returns:
            Self: The session manager instance.
        """
        return self

    async def __aexit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        """Exit the session manager context and cleanup resources.

        Args:
            exc_type: Exception type if an error occurred.
            exc_val: Exception value if an error occurred.
            exc_tb: Exception traceback if an error occurred.
        """
        await self.close()



import asyncio
from collections.abc import AsyncGenerator
from typing import Any

from aiohttp import ClientResponse, ClientSession
import pytest
from pytest_mock import MockerFixture

from aioarxiv.config import ArxivConfig
from aioarxiv.utils.session import SessionManager


@pytest.fixture
def config() -> ArxivConfig:
    """Create a test configuration.

    Returns:
        ArxivConfig: Test configuration with custom timeout and proxy.
    """
    return ArxivConfig(
        timeout=30.0,
        proxy="http://proxy.example.com",
        rate_limit_calls=5,
        rate_limit_period=1.0,
    )


@pytest.fixture
async def mock_session(mocker: MockerFixture) -> AsyncGenerator[Any, None]:
    """Create a mock aiohttp ClientSession.

    Args:
        mocker: pytest-mock fixture

    Yields:
        Any: Mocked ClientSession with async request method.
    """

    def async_return(*args: Any, **kwargs: Any) -> asyncio.Future[Any]:
        future = asyncio.Future()
        future.set_result(None)
        return future

    session = mocker.Mock(spec=ClientSession)
    session.request = mocker.Mock(side_effect=async_return)
    session.close = mocker.Mock(side_effect=async_return)
    session.closed = False
    return session


@pytest.fixture
async def mock_response(mocker: MockerFixture) -> AsyncGenerator[Any, None]:
    """Create a mock aiohttp ClientResponse.

    Args:
        mocker: pytest-mock fixture

    Yields:
        Any: Mocked ClientResponse.
    """
    response = mocker.Mock(spec=ClientResponse)
    future = asyncio.Future()
    future.set_result(response)
    return response


@pytest.mark.asyncio
async def test_basic_request(
    mock_session: Any,
    mock_response: Any,
) -> None:
    """Test basic request functionality.

    Args:
        mock_session: Mocked aiohttp session
        mock_response: Mocked response
    """
    future = asyncio.Future()
    future.set_result(mock_response)
    mock_session.request.return_value = future

    manager = SessionManager(session=mock_session)
    await manager.request("GET", "http://example.com")

    mock_session.request.assert_called_once()
    args, kwargs = mock_session.request.call_args
    assert args[0] == "GET"
    assert args[1] == "http://example.com"


@pytest.mark.asyncio
async def test_request_with_proxy(
    mock_session: Any,
    mock_response: Any,
    config: ArxivConfig,
) -> None:
    """Test request with proxy configuration.

    Args:
        mock_session: Mocked aiohttp session
        mock_response: Mocked response
        config: Test configuration
    """
    future = asyncio.Future()
    future.set_result(mock_response)
    mock_session.request.return_value = future

    manager = SessionManager(session=mock_session, config=config)
    await manager.request("GET", "http://example.com")

    mock_session.request.assert_called_once()
    _, kwargs = mock_session.request.call_args
    assert kwargs.get("proxy") == "http://proxy.example.com"


@pytest.mark.asyncio
async def test_session_lifecycle(
    mock_session: Any,
) -> None:
    """Test session manager lifecycle.

    Args:
        mock_session: Mocked aiohttp session
    """
    async with SessionManager(session=mock_session) as manager:
        assert not mock_session.closed
        await manager.request("GET", "http://example.com")

    mock_session.close.assert_called_once()


@pytest.mark.asyncio
async def test_rate_limiting(
    mocker: MockerFixture,
    mock_response: Any,
) -> None:
    """Test rate limiting behavior.

    Args:
        mocker: pytest-mock fixture
        mock_response: Mocked response

    This test verifies that:
    1. Rate limiting is properly applied
    2. Sleep is called with correct duration
    3. All requests are completed
    """
    # Test parameters
    calls = 2
    period = 1.0
    total_requests = 3

    config = ArxivConfig(
        rate_limit_calls=calls,
        rate_limit_period=period,
    )

    # Mock sleep without actual waiting
    sleep_durations = []

    async def mock_sleep(duration: float) -> None:
        sleep_durations.append(duration)

    mocker.patch("asyncio.sleep", side_effect=mock_sleep)

    # Mock request
    async def mock_request(*args: Any, **kwargs: Any) -> Any:
        return mock_response

    mocker.patch("aiohttp.ClientSession.request", side_effect=mock_request)

    # Execute requests
    async with SessionManager(config=config) as manager:
        tasks = [
            manager.request("GET", "http://example.com") for _ in range(total_requests)
        ]
        await asyncio.gather(*tasks)

    # Verify results
    assert len(sleep_durations) == total_requests - calls, (
        f"Expected {total_requests - calls} rate limit delays, "
        f"got {len(sleep_durations)}"
    )

    # Verify sleep durations
    if sleep_durations:
        assert all(
            0 < d <= period for d in sleep_durations
        ), f"Invalid sleep duration(s): {sleep_durations}"


@pytest.mark.asyncio
async def test_concurrent_requests_limit(config: ArxivConfig) -> None:
    """Test concurrent request limiting.

    Args:
        config: Test configuration
    """
    concurrent_count = 0
    max_concurrent = 0
    semaphore = asyncio.Semaphore(config.rate_limit_calls)

    async with SessionManager(config=config):

        async def test_request() -> None:
            nonlocal concurrent_count, max_concurrent
            async with semaphore:
                concurrent_count += 1
                max_concurrent = max(max_concurrent, concurrent_count)
                await asyncio.sleep(0.01)  # 使用更短的睡眠时间
                concurrent_count -= 1

        tasks = [test_request() for _ in range(10)]
        await asyncio.gather(*tasks)

    assert max_concurrent <= config.rate_limit_calls, (
        f"Max concurrent requests ({max_concurrent}) exceeded "
        f"limit ({config.rate_limit_calls})"
    )



import asyncio
from collections import deque
from dataclasses import dataclass
import functools
from typing import Any, Callable, TypeVar, cast

from .log import logger

T = TypeVar("T", bound=Callable[..., Any])


@dataclass
class RateLimitState:
    """Data class to represent the current rate limit state.

    Args:
        remaining (int): The number of remaining allowed requests in this window.
        reset_at (float): The timestamp (epoch) when the rate-limit window resets.
        window_start (float): The timestamp (epoch) when the current window starts.
    """

    remaining: int
    reset_at: float
    window_start: float


class RateLimiter:
    """A rate limiter that restricts the number of requests within a given time window
    and controls concurrent requests with a semaphore.

    This class uses instance-level locks and semaphores, ensuring each RateLimiter
    instance can manage its own concurrency and request timestamps without
    interfering with other instances or event loops.

    Args:
        calls (int): The maximum number of allowed requests in one time window.
        period (float): The time window length in seconds.

    Usage:
        ```python
        # Create a rate limiter allowing 3 calls every 10 seconds
        limiter = RateLimiter(calls=3, period=10.0)

        # Decorate your async function
        @limiter.limit()
        async def fetch_data(url: str) -> str:
            async with aiohttp.ClientSession() as session:
                async with session.get(url) as response:
                    return await response.text()

        # Use in concurrent operations
        urls = ["http://example.com/1", "http://example.com/2", "http://example.com/3"]
        results = await asyncio.gather(*(fetch_data(url) for url in urls))

        # Use with custom error handling
        async def fetch_with_retry():
            try:
                return await fetch_data("http://example.com")
            except Exception as e:
                logger.error(f"Failed to fetch: {e}")
                return None
        ```
    """

    def __init__(self, calls: int, period: float):
        """Initialize the RateLimiter.

        Args:
            calls (int): The maximum number of allowed requests in one time window.
            period (float): The time window length in seconds.
        """
        self._calls: int = calls
        self._period: float = period
        self._timestamps: deque[float] = deque(maxlen=calls)
        self._lock: asyncio.Lock = asyncio.Lock()
        self._semaphore: asyncio.Semaphore = asyncio.Semaphore(calls)

    def _clean_expired(self, now: float) -> None:
        """Remove timestamps that are outside the current rate limit window.

        Args:
            now (float): The current time (epoch).
        """
        cutoff = now - self._period
        while self._timestamps and self._timestamps[0] <= cutoff:
            self._timestamps.popleft()

    async def _wait_if_needed(self, now: float) -> float:
        """If rate limit is reached, block until a request slot is free.

        Args:
            now (float): The current time (epoch).

        Returns:
            float: Updated current time (after waiting, if needed).
        """
        if len(self._timestamps) >= self._calls:
            wait_time = self._timestamps[0] + self._period - now
            if wait_time > 0:
                logger.debug(
                    f"Rate limit reached, waiting for {wait_time:.2f}s",
                    extra={
                        "wait_time": f"{wait_time:.2f}s",
                        "current_calls": len(self._timestamps),
                        "max_calls": self._calls,
                        "period": self._period,
                    },
                )
                await asyncio.sleep(wait_time)
                # After sleeping, the "now" might have changed
                return asyncio.get_event_loop().time()
        return now

    def limit(self) -> Callable[[T], T]:
        """Decorator for rate limiting an async function.

        This decorator ensures that:
        1. Concurrency does not exceed `calls`
        2. Requests are limited to `calls` within `period` seconds

        Returns:
            Callable[[T], T]: A decorator that wraps the original async function.

        Usage:
            ```python
            limiter = RateLimiter(calls=5, period=1.0)

            @limiter.limit()
            async def api_call(endpoint: str) -> dict:
                async with aiohttp.ClientSession() as session:
                    async with session.get(endpoint) as response:
                        return await response.json()

            # The decorated function will automatically respect rate limits
            result = await api_call("https://api.example.com/data")
            ```
        """

        def decorator(func: T) -> T:
            @functools.wraps(func)
            async def wrapper(*args: Any, **kwargs: Any) -> Any:
                # Acquire semaphore and lock to limit concurrency and ensure timestamp
                # checks are atomic
                async with self._semaphore, self._lock:
                    now = asyncio.get_event_loop().time()
                    self._clean_expired(now)
                    now = await self._wait_if_needed(now)
                    self._timestamps.append(now)

                    logger.debug(
                        "Request rate limit invoked",
                        extra={
                            "current_calls": len(self._timestamps),
                            "max_calls": self._calls,
                            "remaining": self._calls - len(self._timestamps),
                        },
                    )

                # Proceed with the original function call
                return await func(*args, **kwargs)

            return cast(T, wrapper)

        return decorator



from datetime import datetime
from zoneinfo import ZoneInfo

from pydantic import HttpUrl, ValidationError
import pytest
from yarl import URL

from aioarxiv.models import Metadata, Paper, SearchParams, SearchResult


@pytest.fixture
def paper_base_info():
    return {
        "id": "1234.5678",
        "title": "Test",
        "summary": "Test",
        "authors": [{"name": "Test"}],
        "categories": {"primary": {"term": "cs.AI"}, "secondary": []},
        "published": datetime.now(tz=ZoneInfo("Asia/Shanghai")),
        "updated": datetime.now(tz=ZoneInfo("Asia/Shanghai")),
    }


@pytest.mark.parametrize(
    "doi",
    ["10.1234/test.123", "10.12345/test-123", "10.1234/test_123"],
)
def test_valid_doi(paper_base_info, doi) -> None:
    """测试有效的DOI格式"""
    paper = Paper(
        info=paper_base_info,
        doi=doi,
        journal_ref=None,
        pdf_url=None,
        comment=None,
    )
    assert paper.doi == doi


@pytest.mark.parametrize(
    "invalid_doi",
    [
        "11.1234/test.123",  # Not starting with 10
        "10.123/test.123",  # Too few digits
        "test.123",  # Invalid format
    ],
)
def test_invalid_doi(paper_base_info, invalid_doi) -> None:
    """Test paper model with invalid DOI"""
    with pytest.raises(ValidationError):
        Paper(
            info=paper_base_info,
            doi=invalid_doi,
            journal_ref=None,
            pdf_url=None,
            comment=None,
        )


@pytest.mark.parametrize(
    ("start", "end", "expected_seconds", "expected_ms"),
    [
        # Existing cases
        (
            datetime(2024, 1, 1, 12, 0, 0, tzinfo=ZoneInfo("Asia/Shanghai")),
            datetime(2024, 1, 1, 12, 0, 1, tzinfo=ZoneInfo("Asia/Shanghai")),
            1.0,
            1000.0,
        ),
        (
            datetime(2024, 1, 1, 12, 0, 0, tzinfo=ZoneInfo("Asia/Shanghai")),
            datetime(2024, 1, 1, 12, 0, 0, 500000, tzinfo=ZoneInfo("Asia/Shanghai")),
            0.5,
            500.0,
        ),
        (
            datetime(2024, 1, 1, 12, 0, 0, tzinfo=ZoneInfo("Asia/Shanghai")),
            None,
            0.0,
            0.0,
        ),
        (
            datetime(2024, 1, 1, 12, 0, 0, tzinfo=ZoneInfo("Asia/Shanghai")),
            datetime(2024, 1, 1, 12, 0, 0, 1000, tzinfo=ZoneInfo("Asia/Shanghai")),
            0.001,
            1.0,
        ),
    ],
)
def test_metadata_duration(start, end, expected_seconds, expected_ms) -> None:
    """测试元数据持续时间计算"""
    metadata = Metadata(
        start_time=start,
        end_time=end,
        missing_results=0,
        pagesize=20,
        source=URL("http://test.com"),
    )
    assert metadata.duration_seconds == expected_seconds
    assert metadata.duration_ms == expected_ms


@pytest.mark.parametrize("papers_count", [0, 1, 5])
def test_search_result_papers_count(papers_count, paper_base_info) -> None:
    """测试搜索结果论文数量计算"""
    papers = [
        Paper(
            info=paper_base_info,
            doi=None,
            journal_ref=None,
            pdf_url=None,
            comment=None,
        )
        for _ in range(papers_count)
    ]
    result = SearchResult(
        papers=papers,
        total_result=100,
        page=1,
        has_next=True,
        query_params=SearchParams(
            query="test",
            id_list=None,
            start=0,
            max_results=None,
            sort_by=None,
            sort_order=None,
        ),
        metadata=Metadata(
            missing_results=0, pagesize=20, source=URL("http://test.com"), end_time=None
        ),
    )
    assert result.papers_count == papers_count


@pytest.mark.parametrize(
    ("doi", "journal_ref", "pdf_url", "comment"),
    [
        ("10.1234/test.123", "Journal Ref", "https://example.com/pdf", "Test comment"),
        ("10.12345/test-123", None, None, None),
    ],
)
def test_paper_optional_fields(
    paper_base_info, doi, journal_ref, pdf_url, comment
) -> None:
    """Test paper model with optional fields"""
    paper = Paper(
        info=paper_base_info,
        doi=doi,
        journal_ref=journal_ref,
        pdf_url=pdf_url,
        comment=comment,
    )
    assert paper.doi == doi
    assert paper.journal_ref == journal_ref
    if pdf_url is not None:
        assert paper.pdf_url == HttpUrl(pdf_url)
    else:
        assert paper.pdf_url is None
    assert paper.comment == comment
